#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            store.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.7j
#
# Date:            30.01.2017
#
# Description:     Abstract base class for services that can store logged 
#                  data values. 
#                 
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.3)
# 
# Version history: ** 24.02.2009 v0.9a (Olli Lammi) **
#                  First beta version for testers. 
#
#                  ** 26.10.2011 v1.4f (Olli Lammi) **
#                  Added filtering features.
#
#                  ** 07.01.2013 v1.5a (Olli Lammi) **
#                  Added timeval parameter to logging. Added support for
#                  queuing data if storing fails.
#
#                  ** 15.05.2014 v1.6b (Olli Lammi) **
#                  Added clearStagedData-method. Changed added log.Logging
#                  to this class (removed from the implementations). 
#                  Handling and discarding invalid data. 
#
#                  ** 14.01.2015 v1.6e (Olli Lammi) **
#                  Module structure changed.
#
#                  ** 30.01.2017 v1.7j (Olli Lammi) **
#                  Added data point renaming for a data store to filter.
#
###########################################################################

# Imports

import string

from modules.core import threads
from modules.core import configuration
from modules.core import log
from __builtin__ import False


###########################################################################

# Classes

class Store(configuration.Configurable, log.Logging):
    def __init__(self, modname):
        configuration.Configurable.__init__(self)
        log.Logging.__init__(self, modname)
        self.setModuleName(modname)
        self.storefilters = None
        self.staged_data = []
        self.stage_lock = threads.Lock()
    
    def insertData(self, timeval, values):
        if self.storefilters != None:
            temp = []
            for value in values:
                if value[0] in self.storefilters.keys():
                    filter = self.storefilters[value[0]]
                    temp.append([filter, value[1]])
            self.stageData(timeval, temp)
        else:
            self.stageData(timeval, values)
        self.handleStagedData()

    def stageData(self, timeval, values):
        if self.stage_lock.lock_wait():
            self.staged_data.append([timeval, values])
            self.stage_lock.free()

    def handleStagedData(self):
        if self.stage_lock.lock_wait():
            try:
                discarded = []
                inserted = 0
                while len(self.staged_data) > 0:
                    item = self.staged_data[0]
                    self.staged_data.pop(0)
                    if self.insertDataImpl(item[0], item[1]):
                        inserted = 1
                    else:
                        discarded.append(item)
                if inserted:
                    # something of the data went in, try discarded 
                    # items once more, log if not able to insert
                    for item in discarded:
                        if not self.insertDataImpl(item[0], item[1]):
                            self.logDiscardedData(item)
                else:
                    # re-add discarded to stage
                    self.staged_data = discarded
            except:
                pass
            self.stage_lock.free()

    def clearStagedData(self):
        if self.stage_lock.lock_wait():
            while len(self.staged_data) > 0:
                item = self.staged_data[0]
                try:
                    self.logDiscardedData(item)
                except:
                    pass
                self.staged_data.pop(0)
            self.stage_lock.free()
    
    def logDiscardedData(self, data):
        if data == None or len(data) <= 0:
            return
        
        msg = "timestamp:\"%d\"" % data[0]
        for item in data[1]:
            msg = msg + ", %s:\"%s\"" % (item[0], item[1])
        
        self.Log("Discarding staged data: " + msg)
    
    def initStoreFilters(self, conf):
        self.storefilters = None
        temp = conf.getValue('STOREFILTER', [])
        if len(temp) > 0:
            tempfilters = {}
            for filter in temp:
                mname = None
                posname = None
                storepos = None
                if string.find(filter, ':') < 0:
                    return False
                tempnames = string.split(filter, ':')
                if len(tempnames) == 2:
                    mname = string.strip(tempnames[0])
                    posname = string.strip(tempnames[1])
                    storepos = posname
                elif len(tempnames) == 3:
                    mname = string.strip(tempnames[0])
                    posname = string.strip(tempnames[1])
                    storepos = string.strip(tempnames[2])
                else:
                    return False
                if mname == self.getModuleName() and posname not in tempfilters.keys():
                    tempfilters[posname] = storepos
            if len(tempfilters) > 0:
                self.storefilters = tempfilters
        return True

    def checkStoreFilters(self, measurepositions):
        if self.storefilters != None:
            for filter in self.storefilters.keys():
                if filter not in measurepositions:
                    return (0, "Data store module %s has invalid store filter %s." % (self.getModuleName(), filter))
        return (1, '')

    # returns 0 if inserting failed, 1 with success
    def insertDataImpl(self, timeval, values): pass
    
