#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            enerventEda.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#                  (Enervent EDA datapoint table originally from 
#                  http://www.enervent.fi/data/freeway/EDA_Modbus_Registers_2011_09_14.pdf)
#
# Version:         0.1b
#
# Date:            01.02.2017
#
# Description:     Library to communicate with the Enervent EDA devices.
#                  
# Supported devices: 
#                  Enervent EDA devices.
#
# Requirements:    Python interpreter 2.6 or newer (www.python.org)
#
# Version history: ** 03.02.2015 v0.1a (Olli Lammi) **
#                  First test version. 
#
#                  ** 03.02.2015 v0.1a (Olli Lammi) **
#                  Added configurable off-by-one fix for serial listener. 
#
###########################################################################

# Imports

import sys, os 
import string
import time 

from modules.core import threads
from modules.core import log
from modules.utils import lockfile
from modules.utils import binary

from modules.utils import modbus

###########################################################################

# Constants

BAUDRATE = 19200
# 8bit, parity none, modbus unitid 1

DATAVALID = 240

###########################################################################

# Device configuration data

TYPE_COIL = 1
TYPE_HREG = 3

TYPE_UINT8 = 1
TYPE_UINT16 = 2
TYPE_INT16 = 3
TYPE_INT16_10 = 4
TYPE_INT16_1000 = 5

TYPE_R = 1
TYPE_W = 2
TYPE_RW = 3

EDA_DEVICES = { 'DEFAULT': [ \
        ['STOP', 0, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Fans stopped, stop 1 / run 0'], \
        ['Away', 1, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Away function, on / off'], \
        ['Long away', 2, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Long away function, on / off'], \
        ['Overpressure', 3, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Over pressure, on / off'], \
        ['Cooker hood', 4, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Cooker hood, on / off'], \
        ['Central vacuum cleaner', 5, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Central vacuum cleaner, on / off'], \
        ['Max heating', 6, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Max heating function, on / off'], \
        ['Max cooling', 7, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Max cooling function, on / off'], \
        ['CO2 boost', 8, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'CO2 boost, allowed 1 / not allowed 0'], \
        ['Humidity boost allow', 9, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Humidity boost, allowed 1 / not allowed 0'], \
        ['Manual boost', 10, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Manual boosting, on / off'], \
        ['Temperature boost', 11, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Temperature boost allowed 1 / not allowed 0'], \
        ['Summernight cooling', 12, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Summernight cooling, allowed 1 / not allowed 0'], \
        ['Heating pump summer use', 13, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Heating circulation pump summer use, allowed 1 / not allowed 0'], \
        ['Heat recovery scaling', 14, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Heat recovery voltage scaling, allowed 1 / not allowed 0'], \
        ['If cooling is not allowed during summer night cooling', 15, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Blockage for active cooling during Summer night cooling, allowed 1 / not allowed 0'], \
        ['Fan type', 16, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Fan control type, EC 1 / AC 0'], \
        ['Humidity boost', 17, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Humidity boost, with fixed limit 1 / automatic 0'], \
        ['After heating on/off in away function', 18, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Is after heating used in away function, allowed 1 / not allowed 0'], \
        ['Cooling on/off in away function', 19, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Is extra cooling used in away function, allowed 1 / not allowed 0'], \
        ['After heating on/off in long away function', 20, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Is after heating used in long away function, allowed 1/ not allowed 0'], \
        ['Cooling on/off in long away function', 21, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Is extra cooling used in long away function, allowed 1 / not allowed 0'], \
        ['DDC control', 22, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'DDC control, manual control 1 / auto 0'], \
        ['Fan control', 23, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Fan control type, constant duct pressure 1 / speed control 0'], \
        ['Pressure switch', 26, TYPE_COIL, TYPE_UINT8, TYPE_R, 'Electrical heater pressure switch data, pressure switch closed 1 / open 0'], \
        ['Cooling error data', 27, TYPE_COIL, TYPE_UINT8, TYPE_R, 'Cooling error data, closed 1 / open 0'], \
        ['Cooling indication', 28, TYPE_COIL, TYPE_UINT8, TYPE_R, 'Cooling indication, closed 1 / open 0'], \
        ['Heat recovery error', 29, TYPE_COIL, TYPE_UINT8, TYPE_R, 'Heat recovery error data, closed 1 / open 0'], \
        ['Heat recovery indication', 30, TYPE_COIL, TYPE_UINT8, TYPE_R, 'Heat recovery indication, closed 1 / open 0'], \
        ['Heating error', 31, TYPE_COIL, TYPE_UINT8, TYPE_R, 'Heating error data, closed 1 / open 0'], \
        ['Heating indication', 32, TYPE_COIL, TYPE_UINT8, TYPE_R, 'Heating indication, closed 1 / open 0'], \
        ['EH control', 33, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Electric heater control type of closing contact, fault idication 1 / run indication 0'], \
        ['External heating disable', 34, TYPE_COIL, TYPE_UINT8, TYPE_R, 'External heating disabled control, closed 1 / open 0'], \
        ['External cooling disable', 35, TYPE_COIL, TYPE_UINT8, TYPE_R, 'External cooling disabled control, closed 1 / open 0'], \
        ['AI Temperature decrease', 36, TYPE_COIL, TYPE_UINT8, TYPE_R, 'Temperature decrease Active 1 / Inactive 0'], \
        ['DI extended time', 37, TYPE_COIL, TYPE_UINT8, TYPE_R, 'DI extended time, closed 1 / open 0'], \
        ['ES circuit control', 38, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Emergency stop circuit type, active when closed (NO) 1 / active when open (NC) 0'], \
        ['Fire risk control', 39, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Fire risk circuit type, active when closed (NO) 1 / active when open (NC) 0'], \
        ['Alarm A', 41, TYPE_COIL, TYPE_UINT8, TYPE_R, 'A alarm data'], \
        ['Alarm B', 42, TYPE_COIL, TYPE_UINT8, TYPE_R, 'B alarm data'], \
        ['Time program', 43, TYPE_COIL, TYPE_UINT8, TYPE_R, 'Time program in use, active 1 / non active 0'], \
        ['Real time ventilation speed indication', 44, TYPE_COIL, TYPE_UINT8, TYPE_R, 'Real time ventilation speed shown in display or not'], \
        ['Control step indication', 45, TYPE_COIL, TYPE_UINT8, TYPE_R, 'Heating or cooling possible, heating possible 1 / cooling possible 0'], \
        ['External defrosting indication EDX', 46, TYPE_COIL, TYPE_UINT8, TYPE_R, 'EDX outside unit in defrosting condition, defrosting 1 / not defrosting 0'], \
        ['EH cooling', 48, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Electrical heater cooldown in Stop mode, in use 1 / not in use 0'], \
        ['Service reminder', 49, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Service reminder, allowed 1 / not allowed 0'], \
        ['Freeze risk indication', 50, TYPE_COIL, TYPE_UINT8, TYPE_R, 'Freeze risk indication, freeze risk 1 / reset 0'], \
        ['Unit type', 51, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Unit type Pro 1 / Family 0'], \
        ['Cooling', 52, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Cooling allowed by unit, yes 1 / no 0'], \
        ['HR', 53, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Heat recovery intalled in unit (in use), yes 1 / no 0'], \
        ['Heating', 54, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Additional heating allowed by unit (additional heating installed), yes 1 / no 0'], \
        ['Defrosting of heat recovery', 55, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Defrosting function of heat recovery, allowed 1 / not allowed 0'], \
        ['OP1', 56, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Operating panel 1 in use for temperature measures'], \
        ['OP2', 57, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Operating panel 2 in use for temperature measures'], \
        ['OP3', 58, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Operating panel 3 in use for temperature measures'], \
        ['OP4', 59, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Operating panel 4 in use for temperature measures'], \
        ['OP5', 60, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Operating panel 5 in use for temperature measures'], \
        ['TEMP_TRANS_1', 61, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Temperature transmitter 1 in use for temperature measures'], \
        ['TEMP_TRANS_2', 62, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Temperature transmitter 2 in use for temperature measures'], \
        ['TEMP_TRANS_3', 63, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Temperature transmitter 3 in use for temperature measures'], \
        ['OVER_PRESSURE', 64, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Over pressure in use for quick functions'], \
        ['BOOSTING', 65, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Boosting in use for quick functions'], \
        ['AWAY', 66, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Away use for quick functions'], \
        ['LONG AWAY', 67, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Long away in use for quick functions'], \
        ['MAX HEAT / COOL', 68, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Max heating / cooling in use for quick functions'], \
        ['NIGHT COOL', 69, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Night cool in use for quick functions'], \
        ['FANSPEED', 70, TYPE_COIL, TYPE_UINT8, TYPE_RW, 'Fan speed selection available in main screen'], \
        ['TEMP SP', 71, TYPE_COIL, TYPE_INT16_10, TYPE_RW, 'Temperature selection available in main screen'], \
        ['OP1 temperature', 1, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Display 1 temperature'], \
        ['OP2 temperature', 2, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Display 2 temperature'], \
        ['OP3 temperature', 3, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Display 3 temperature'], \
        ['OP4 temperature', 4, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Display 4 temperature'], \
        ['OP5 temperature', 5, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Display 5 temperature'], \
        ['Fresh air temperature X1', 6, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Fresh air temperature measurement. Register value = temperature measurement * 10!'], \
        ['Supply air temperature after heat recovery X2', 7, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Supply air temperature measurement after heat recovery. Register value = temperature measurement * 10!'], \
        ['Supply air temperature X3', 8, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Supply air temperature measurement. Register value = temperature measurement * 10!'], \
        ['Waste air temperature X4', 9, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Waste air temperature measurement. Register value = temperature measurement * 10!'], \
        ['Exhaust air temperature X5', 10, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Exhaust air temperature measurement. Register value = temperature measurement * 10!'], \
        ['Exhaust air temperature before heat recovery X8', 11, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Exhaust air temperature measurement before heat recovery. Register value = temperature measurement *10!'], \
        ['Return water temperature X8', 12, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Return water temperature measurement. Register value = temperature measurement * 10!'], \
        ['Exhaust air humidity X5', 13, TYPE_HREG, TYPE_INT16, TYPE_R, 'Exhaust air humidity measurement'], \
        ['Supply air filter Pa', 14, TYPE_HREG, TYPE_INT16, TYPE_R, 'Pressure difference over supply air filter X6'], \
        ['Exhaust air filter Pa', 15, TYPE_HREG, TYPE_INT16, TYPE_R, 'Pressure difference over exhaust air filter X6'], \
        ['Heat recovery Pa', 16, TYPE_HREG, TYPE_INT16, TYPE_R, 'Pressure difference over heat recovery X6'], \
        ['AI1', 17, TYPE_HREG, TYPE_INT16, TYPE_R, 'Measured Analog input 1 value'], \
        ['AI2', 18, TYPE_HREG, TYPE_INT16, TYPE_R, 'Measured Analog input 2 value'], \
        ['AI3', 19, TYPE_HREG, TYPE_INT16, TYPE_R, 'Measured Analog input 3 value'], \
        ['AI4', 20, TYPE_HREG, TYPE_INT16, TYPE_R, 'Measured Analog input 4 value'], \
        ['AI5', 21, TYPE_HREG, TYPE_INT16, TYPE_R, 'Measured Analog input 5 value'], \
        ['AI6', 22, TYPE_HREG, TYPE_INT16, TYPE_R, 'Measured Analog input 6 value'], \
        ['AI1 RES', 23, TYPE_HREG, TYPE_INT16, TYPE_R, 'Calculated Analog input 1 value'], \
        ['AI2 RES', 24, TYPE_HREG, TYPE_INT16, TYPE_R, 'Calculated Analog input 2 value'], \
        ['AI3 RES', 25, TYPE_HREG, TYPE_INT16, TYPE_R, 'Calculated Analog input 3 value'], \
        ['AI4 RES', 26, TYPE_HREG, TYPE_INT16, TYPE_R, 'Calculated Analog input 4 value'], \
        ['AI5 RES', 27, TYPE_HREG, TYPE_INT16, TYPE_R, 'Calculated Analog input 5 value'], \
        ['AI6 RES', 28, TYPE_HREG, TYPE_INT16, TYPE_R, 'Calculated Analog input 6 value'], \
        ['Heat recovery supply side %', 29, TYPE_HREG, TYPE_INT16, TYPE_R, 'Efficiency of heat recovery on supply side'], \
        ['Heat recovery exhaust side %', 30, TYPE_HREG, TYPE_INT16, TYPE_R, 'Efficiency of heat recovery on exhaust side'], \
        ['Heat recovery temperature difference supply side', 31, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Heat recovery temp. difference on supply side'], \
        ['Heat recovery temp difference exhaust side', 32, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Heat recovery temp. Difference on exhaust side'], \
        ['Supply air coil delta t', 33, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Supply air coil temperature difference'], \
        ['Exhaust fan temperature difference', 34, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Exhaust fan temperature difference'], \
        ['48h Rh measurement', 35, TYPE_HREG, TYPE_INT16, TYPE_R, '48h relative humidity level mean value'], \
        ['Absolute humidity', 36, TYPE_HREG, TYPE_INT16, TYPE_R, 'Absolute humidity of exhaust air, calculated'], \
        ['Sec clock', 37, TYPE_HREG, TYPE_INT16, TYPE_R, 'RTC circuit, clock seconds'], \
        ['Min clock', 38, TYPE_HREG, TYPE_INT16, TYPE_R, 'RTC circuit, clock minutes'], \
        ['Hour clock', 39, TYPE_HREG, TYPE_INT16, TYPE_R, 'RTC circuit, clock hours'], \
        ['Day', 40, TYPE_HREG, TYPE_INT16, TYPE_R, 'RTC circuit, clock date'], \
        ['Month', 41, TYPE_HREG, TYPE_INT16, TYPE_R, 'RTC circuit, clock month'], \
        ['YEAR', 42, TYPE_HREG, TYPE_INT16, TYPE_R, 'RTC circuit, clock year. Year = 2000 + register value'], \
        ['Weekday', 43, TYPE_HREG, TYPE_INT16, TYPE_R, 'Weekday, Mon=1, Tue=2, ... Sun=0'], \
        ['State data', 44, TYPE_HREG, TYPE_UINT16, TYPE_R, 'Here you can see in which condition the unit is, 0=Normal condition, 1=Max cooling, 2=Max heating, 4=emergency stop, 8=stop, 16=away, 32=long away, 46=temperature boosting, 128=CO2 boosting, 256=Rh boosting, 512=Boosting, 1024=over pressure, 2048=Cooker hood, 4096=Central vacuum cleaner, 8192=ELH cooling, 16384=Summernight cooling, 32768=EDX defrosting. If 2 or more states are active the register value is the sum of these states. LSB'], \
        ['Control steps of temperature', 45, TYPE_HREG, TYPE_INT16, TYPE_R, 'Here you can see the condition of temperature control. 0=nothing used, 1=cooling, 2=heat recovery, 4=heating, 5=step delay state, 6=Summer night cooling, 7=Startup, 8=Stop, 9=HR clean, 10=EXT unit defrost'], \
        ['Room temperature mean value', 46, TYPE_HREG, TYPE_INT16, TYPE_R, 'Room temperature calculated from one or more room sensors'], \
        ['Cascade SP', 47, TYPE_HREG, TYPE_INT16, TYPE_R, 'Set value of cascade control'], \
        ['Cascade P', 48, TYPE_HREG, TYPE_INT16, TYPE_R, 'P-value of cascade control'], \
        ['Cascade I', 49, TYPE_HREG, TYPE_INT16, TYPE_R, 'I-value of cascade control'], \
        ['Actual ventilation level', 50, TYPE_HREG, TYPE_INT16, TYPE_R, 'Value of ventilation level affected by external control'], \
        ['SUPPLY fan basic speed', 51, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Fan speed difference setting, Supply fan value, if AC = 1-8 (register 1x0016=0), DC = 20-100 (register 1x0016=1)'], \
        ['EXHAUST fan basic speed', 52, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Fan speed difference setting, Exhaust fan value, if AC = 1-8 (register 1x0016=0), DC = 20-100 (register 1x0016=1)'], \
        ['Ventilation level selected from user panel', 53, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Fanspeed selection, if AC = 1-8 register 1x0016=0), DC = 20-100 (register 1x0016=1)'], \
        ['Supply fan overpressure', 54, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Supply fan speed during overpressure'], \
        ['Exhaust fan overpressure', 55, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Exhaust fan speed during overpressure'], \
        ['Overpressure time left', 56, TYPE_HREG, TYPE_INT16, TYPE_R, 'Remaining time for overpressure state'], \
        ['Overpressure time', 57, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Overpressure expected time / delay (min)'], \
        ['Supply Cooker hood', 58, TYPE_HREG, TYPE_INT16, TYPE_RW, 'SUPPLY fan speed, cooker hood in use'], \
        ['Exhaust Cooker hood', 59, TYPE_HREG, TYPE_INT16, TYPE_RW, 'EXHAUST fan speed, cooker hood in use'], \
        ['Supply Central vacuum cleaner', 60, TYPE_HREG, TYPE_INT16, TYPE_RW, 'SUPPLY fan speed, central vacuum cleaner in use'], \
        ['Exhaust Central vacuum cleaner', 61, TYPE_HREG, TYPE_INT16, TYPE_RW, 'EXHAUST fan speed, central vacuum cleaner in use'], \
        ['Supply CH CVC, OVP,CVC, OVP, CH', 62, TYPE_HREG, TYPE_INT16, TYPE_RW, 'SUPPLY fan speed, central vacuum cl. and cooker hood or overpressure and central VC or overpressure and cooker hood in use'], \
        ['Exhaust CH CVC, OVP CVC, OVP CH', 63, TYPE_HREG, TYPE_INT16, TYPE_RW, 'EXHAUST fan speed, central vacuum cl. and cooker hood or overpressure and central VC or overpressure and cooker hood in use'], \
        ['Supply OVP, CVC, CH in use', 64, TYPE_HREG, TYPE_INT16, TYPE_RW, 'SUPPLY fan speed overpressure and central VC and cooker hood in use'], \
        ['Exhaust OVP, CVC, CH in use', 65, TYPE_HREG, TYPE_INT16, TYPE_RW, 'EXHAUST fan speed overpressure and central VC and cooker hood in use'], \
        ['Manual boosting time', 66, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Manual boosting time / delay (min)'], \
        ['Level during boosting', 67, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Ventilation level during boosting (smaller if there is a difference between fans)'], \
        ['Changeable Boosting ventilation speed', 68, TYPE_HREG, TYPE_INT16, TYPE_R, 'When Boosting is active, you can change ventilation speed from control panel to a higher level trough this register'], \
        ['RH boosting limit fixed', 69, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Fixed relative humidity boosting, switching limit value (%)'], \
        ['RH boosting limit automatic', 70, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Automatic relative humidity boosting switching limit value (%)'], \
        ['RH boosting P value', 71, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Relative humidity boosting, controller proportional band (%)'], \
        ['RH boosting Reset time', 72, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Relative humidity, controller reset time (min)'], \
        ['RH boosting Integration time', 73, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Relative humidity, controller integration time (min)'], \
        ['RH boosting max ventilation', 74, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Max ventilation level during relative humidity boosting'], \
        ['RH boosting Dz', 75, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Relative humidity boosting, controller dead zone (%)'], \
        ['CO2 boosting limit', 76, TYPE_HREG, TYPE_INT16, TYPE_RW, 'CO2 boosting, switching limit value (ppm)'], \
        ['CO2 boosting max ventilation', 77, TYPE_HREG, TYPE_INT16, TYPE_RW, 'max ventilation level during CO2 boosting'], \
        ['CO2 boosting P value', 78, TYPE_HREG, TYPE_INT16, TYPE_RW, 'CO2 boosting controller proportional band (ppm)'], \
        ['CO2 boosting Reset time', 79, TYPE_HREG, TYPE_INT16, TYPE_RW, 'CO2 boosting controller reset time (min)'], \
        ['CO2 boosting Integration time', 80, TYPE_HREG, TYPE_INT16, TYPE_RW, 'CO2 boosting controller integration time (min)'], \
        ['CO2 boosting Dz', 81, TYPE_HREG, TYPE_INT16, TYPE_RW, 'CO2 boosting controller deadzone (ppm)'], \
        ['Temperature boosting, measuring sensor', 82, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Temperature boosting, measuring sensor, 1=control panel 1, 2=control panel 2, 3=control panel 3, 4=control panel 4, 5=control panel 5, 6=temperature transmitter 1, 7=temperature transmitter 2, 8=temperature transmitter 3, 9=temp. transmitter of exhaust air'], \
        ['Temperature boosting, max ventilation', 83, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Max ventilation level of temperature boosting'], \
        ['Temperature boosting, P value', 84, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Temperature boosting, controller proportional band (C) (50=5.0)'], \
        ['Temperature boosting, Reset time', 85, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Temperature boosting, controller reset time (min)'], \
        ['Temperature boosting, Integration time', 86, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Temperature boosting, controller integration time (min)'], \
        ['Temperature boosting, Dz', 87, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Temperature boosting, controller dead zone (C) (5=0.5)'], \
        ['Temperature restriction, P value', 88, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Temperature restriction, proportional band (C)'], \
        ['Temperature restriction, Reset time', 89, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Temperature restriction, antiwindup reset time (min)'], \
        ['Temperature restriction, Integration time', 90, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Temperature restriction integration time (min)'], \
        ['Temperature restriction, Dz', 91, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Temperature restriction deadzone (C)'], \
        ['SNC ventilation level', 92, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Summernight cooling level of ventilation'], \
        ['SNC outside temperature', 93, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Summernight cooling min. outside temperature below which SNC doesnt switch on (C)'], \
        ['SNC start', 94, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'SNC on temperature, if room or exhaust air temperature is above this value the function will switch on (C)'], \
        ['SNC stop', 95, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'SNC stop temperature, if room or exhaust air temperature falls below this value the function will switch off (C)'], \
        ['SNC diff', 96, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'SNC cooling difference, difference between outside temp or room/exhaust temp must be above this value so that function is on (C)'], \
        ['SNC week', 97, TYPE_HREG, TYPE_INT16, TYPE_RW, 'SNC cooling, weekdays when function is on sun=1 mon=2, tue=4, wed=8, thu=16, fri=32, sat=64 and combinations are summed up'], \
        ['SNC hour start', 98, TYPE_HREG, TYPE_INT16, TYPE_RW, 'SNC cooling switched on, hours'], \
        ['SNC hour stop', 99, TYPE_HREG, TYPE_INT16, TYPE_RW, 'SNC cooling stopped, hours'], \
        ['Ventilation level during away-function', 100, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Ventilation level during away-function'], \
        ['Away temperature drop', 101, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Away function, drop of requested temperature (C)'], \
        ['Ventilation level of long away-function', 102, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Ventilation level under long away -function'], \
        ['Long away, temperature drop', 103, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Long away function, drop of requested temperature (C)'], \
        ['AI1 TYPE', 104, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 1 input type, 0=NA, 1=CO2_1, 2=CO2_2, 3=CO2_3, 4=RH_1, 5=RH_2, 6=RH_3, 7=OUT_TERM, 8=ROOM_TERM_1, 9=ROOM_TERM_2, 10=ROOM_TERM_3, 11=TEMP_SP, 12=Time relay, 13=External heating disable, 14=External cooling disable, 15=PDE10, 16=PDE30'], \
        ['AI2 TYPE', 105, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 2 input type, 0=NA, 1=CO2_1, 2=CO2_2, 3=CO2_3, 4=RH_1, 5=RH_2, 6=RH_3, 7=OUT_TERM, 8=ROOM_TERM_1, 9=ROOM_TERM_2, 10=ROOM_TERM_3, 11=TEMP_SP, 12=Time relay, 13=External heating disable, 14=External cooling disable, 15=PDE10, 16=PDE30'], \
        ['AI3 TYPE', 106, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 3 input type, 0=NA, 1=CO2_1, 2=CO2_2, 3=CO2_3, 4=RH_1, 5=RH_2, 6=RH_3, 7=OUT_TERM, 8=ROOM_TERM_1, 9=ROOM_TERM_2, 10=ROOM_TERM_3, 11=TEMP_SP, 12=Time relay, 13=External heating disable, 14=External cooling disable, 15=PDE10, 16=PDE30'], \
        ['AI4 TYPE', 107, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 4 input type, 0=NA, 1=CO2_1, 2=CO2_2, 3=CO2_3, 4=RH_1, 5=RH_2, 6=RH_3, 7=OUT_TERM, 8=ROOM_TERM_1, 9=ROOM_TERM_2, 10=ROOM_TERM_3, 11=TEMP_SP, 12=Time relay, 13=External heating disable, 14=External cooling disable, 15=PDE10, 16=PDE30'], \
        ['AI5 TYPE', 108, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 5 input type, 0=NA, 1=CO2_1, 2=CO2_2, 3=CO2_3, 4=RH_1, 5=RH_2, 6=RH_3, 7=OUT_TERM, 8=ROOM_TERM_1, 9=ROOM_TERM_2, 10=ROOM_TERM_3, 11=TEMP_SP, 12=Time relay, 13=External heating disable, 14=External cooling disable, 15=PDE10, 16=PDE30'], \
        ['AI6 TYPE', 109, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 6 input type, 0=NA, 1=CO2_1, 2=CO2_2, 3=CO2_3, 4=RH_1, 5=RH_2, 6=RH_3, 7=OUT_TERM, 8=ROOM_TERM_1, 9=ROOM_TERM_2, 10=ROOM_TERM_3, 11=TEMP_SP, 12=Time relay, 13=External heating disable, 14=External cooling disable, 15=PDE10, 16=PDE30'], \
        ['AI1 VL', 110, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'AI 1 voltage lower limit, 100 = 10.0V'], \
        ['AI2 VL', 111, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'AI 2 voltage lower limit, 100 = 10.0V'], \
        ['AI3 VL', 112, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'AI 3 voltage lower limit, 100 = 10.0V'], \
        ['AI4 VL', 113, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'AI 4 voltage lower limit, 100 = 10.0V'], \
        ['AI5 VL', 114, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'AI 5 voltage lower limit, 100 = 10.0V'], \
        ['AI6 VL', 115, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'AI 6 voltage lower limit, 100 = 10.0V'], \
        ['AI1 VH', 116, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'AI 1 voltage upper limit, 100 = 10.0V'], \
        ['AI2 VH', 117, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'AI 2 voltage upper limit, 100 = 10.0V'], \
        ['AI3 VH', 118, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'AI 3 voltage upper limit, 100 = 10.0V'], \
        ['AI4 VH', 119, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'AI 4 voltage upper limit, 100 = 10.0V'], \
        ['AI5 VH', 120, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'AI 5 voltage upper limit, 100 = 10.0V'], \
        ['AI6 VH', 121, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'AI 6 voltage upper limit, 100 = 10.0V'], \
        ['AI1 RL', 122, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 1 result lower limit'], \
        ['AI2 RL', 123, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 2 result lower limit'], \
        ['AI3 RL', 124, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 3 result lower limit'], \
        ['AI4 RL', 125, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 4 result lower limit'], \
        ['AI5 RL', 126, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 5 result lower limit'], \
        ['AI6 RL', 127, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 6 result lower limit'], \
        ['AI1 RH', 128, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 1 result upper limit'], \
        ['AI2 RH', 129, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 2 result upper limit'], \
        ['AI3 RH', 130, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 3 result upper limit'], \
        ['AI4 RH', 131, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 4 result upper limit'], \
        ['AI5 RH', 132, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 5 result upper limit'], \
        ['AI6 RH', 133, TYPE_HREG, TYPE_INT16, TYPE_RW, 'AI 6 result upper limit'], \
        ['CAS R t', 134, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Reset time for cascade controller (min)'], \
        ['Temperature setpoint', 135, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'temperature request for controlling form in question. Register value = setting * 10!'], \
        ['Temperature control mode', 136, TYPE_HREG, TYPE_INT16, TYPE_RW, 'The way to control the temperature of ventilation unit, 1=Supply control, 2=Exhaust control, 3=Room control'], \
        ['SPLY T MIN', 138, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Supply air temperature control minimum limit (C) (50=5.0)'], \
        ['SPLY T MAX', 139, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Supply air temperature control maximum limit (C) (50=5.0)'], \
        ['SP MIN', 140, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Min value of requested temperature set from display'], \
        ['SP MAX', 141, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Max value of requested temperature set from display'], \
        ['SupplyC P value', 142, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Supply air temperature controller propotional band (C) (50=5.0)'], \
        ['SupplyC Reset time', 143, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Supply air temperature controller reset time'], \
        ['SupplyC Integration time', 144, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Supply air temperature controller integration time (min)'], \
        ['SupplyC Dz', 145, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Supply air temperature controller dead zone (C) (50=5.0)'], \
        ['STEP DELAY', 146, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Delay between control steps (sec)'], \
        ['Cool P value', 147, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Cooling step proportional band (C) (50=5.0)'], \
        ['Cool Integration time', 148, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Cooling step integration time (sec)'], \
        ['HR P value', 149, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'HR step proportional band (C) (50=5.0)'], \
        ['HR Integration time', 150, TYPE_HREG, TYPE_INT16, TYPE_RW, 'HR step integration time (sec)'], \
        ['Heat P value', 151, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Heat step proportional band (C) (50=5.0)'], \
        ['Heat Integration time', 152, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Heat step integration time (sec)'], \
        ['HP DELAY', 153, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Heat pump min run time (min)'], \
        ['COOL TYPE', 154, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Selection of cooling type, 0=No cooling, 1=CW, 2=HP, 3=CG, 4=CX, 5=CX_INV, 6=X2CX, 7=CXBIN, 8=Cooler'], \
        ['COOL QRY1 SET', 155, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Cooling request 1 on (%)'], \
        ['COOL QRY1 RESET', 156, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Cooling request 1 off (%), always smaller than COOL QRY1 SET'], \
        ['COOL QRY2 SET', 157, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Cooling request 2 on (%)'], \
        ['COOL QRY2 RESET', 158, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Cooling request 2 off (%), always smaller than COOL QRY2 SET'], \
        ['COOL QRY3 SET', 159, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Cooling request 3 on (%)'], \
        ['COOL QRY3 RESET', 160, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Cooling request 3 off (%), always smaller than COOL QRY3 SET'], \
        ['COOL RSTART DEL', 161, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Restart delay for cooling step 1 (min)'], \
        ['COOL BLOCK T', 164, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Block limit of cooling, cooling blocked when outside temperature is below this value (C) (50=5.0)'], \
        ['LTO25', 165, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'voltage value of heat recovery voltage scaling for 25% point (V) (100=10V)'], \
        ['LTO50', 166, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'heat recovery, voltage value of voltage scaling for 50% point (V) (100=10V)'], \
        ['LTO75', 167, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'heat recovery, voltage value of voltage scaling for 75% point (V) (100=10V)'], \
        ['LTO FREEZE', 168, TYPE_HREG, TYPE_INT16, TYPE_RW, 'heat recovery de-frost, pressure switch defrosting limit (Pa)'], \
        ['LTO FREEZE DEL', 169, TYPE_HREG, TYPE_INT16, TYPE_RW, 'heat recovery de-frost, delay setting (min)'], \
        ['LTO FREEZE T', 170, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'heat recovery de-frost temperature (C) (50=5.0)'], \
        ['HEAT TYPE', 171, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Selection of heating type, 0=No heater, 1=WBU, 2=HP, 3=EH, 4=EH PWM'], \
        ['TEMP DEC', 172, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Temperature drop, 0= not in use. Selection with a year timer/week timer or DI input (C) (50=5.0)'], \
        ['WBU FOREC', 173, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Returnwater anticipation safety temerature in water heated units (C) (50=5.0)'], \
        ['T WR SP', 174, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Returnwater temperature setpoint in Stop mode (C) (50=5.0)'], \
        ['T WR P', 175, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Returnwater control proportional band in stop mode (C) (50=5.0)'], \
        ['START T 0', 176, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Controls at 0 if outside temperature exceeds this temperature during startup (C) (50=5.0)'], \
        ['START T 1', 177, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'HR at max if outside air exceeds this temperature during startup (C) (50=5.0)'], \
        ['START T 2', 178, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Heat at max if outside air exceeds this temperature during startup (C) (50=5.0)'], \
        ['REWA START', 179, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Returnwater setpoint during startup (C) (50=5.0)'], \
        ['EXT DEL', 180, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Exhaust fan startup delay, time after damper delay (sec)'], \
        ['EXT START', 181, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Exhaust fan speed during startup'], \
        ['SPLY START', 182, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Supply fan speed during startup'], \
        ['PELTI DEL', 183, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Damper delay (sec)'], \
        ['START T', 184, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Startup delay (sec)'], \
        ['DI3 TYPE', 185, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital input 3 function, 0=No function, 1=Away, 2=Long away, 3=Overpressure, 4=Cooker hood, 5=Central vacuum cleaner, 6=Max heating, 7=Max cooling, 10=Boosting, 26=EH PDA, 27=Cooling error, 28=Cooling run indication, 29=HR error, 30=HR run indicaion, 34=External heating disable, 35=External cooling disable, 36=AI temperature drop, 37=Time extension, 45=EXT defrost'], \
        ['DI4 TYPE', 186, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital input 4 function, 0=No function, 1=Away, 2=Long away, 3=Overpressure, 4=Cooker hood, 5=Central vacuum cleaner, 6=Max heating, 7=Max cooling, 10=Boosting, 26=EH PDA, 27=Cooling error, 28=Cooling run indication, 29=HR error, 30=HR run indicaion, 34=External heating disable, 35=External cooling disable, 36=AI temperature drop, 37=Time extension, 45=EXT defrost'], \
        ['DI5 TYPE', 187, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital input 5 function, 0=No function, 1=Away, 2=Long away, 3=Overpressure, 4=Cooker hood, 5=Central vacuum cleaner, 6=Max heating, 7=Max cooling, 10=Boosting, 26=EH PDA, 27=Cooling error, 28=Cooling run indication, 29=HR error, 30=HR run indicaion, 34=External heating disable, 35=External cooling disable, 36=AI temperature drop, 37=Time extension, 45=EXT defrost'], \
        ['DI6 TYPE', 188, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital input 6 function, 0=No function, 1=Away, 2=Long away, 3=Overpressure, 4=Cooker hood, 5=Central vacuum cleaner, 6=Max heating, 7=Max cooling, 10=Boosting, 26=EH PDA, 27=Cooling error, 28=Cooling run indication, 29=HR error, 30=HR run indicaion, 34=External heating disable, 35=External cooling disable, 36=AI temperature drop, 37=Time extension, 45=EXT defrost'], \
        ['DI7 TYPE', 189, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital input 7 function, 0=No function, 1=Away, 2=Long away, 3=Overpressure, 4=Cooker hood, 5=Central vacuum cleaner, 6=Max heating, 7=Max cooling, 10=Boosting, 26=EH PDA, 27=Cooling error, 28=Cooling run indication, 29=HR error, 30=HR run indicaion, 34=External heating disable, 35=External cooling disable, 36=AI temperature drop, 37=Time extension, 45=EXT defrost'], \
        ['DI8 TYPE', 190, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital input 8 function, 0=No function, 1=Away, 2=Long away, 3=Overpressure, 4=Cooker hood, 5=Central vacuum cleaner, 6=Max heating, 7=Max cooling, 10=Boosting, 26=EH PDA, 27=Cooling error, 28=Cooling run indication, 29=HR error, 30=HR run indicaion, 34=External heating disable, 35=External cooling disable, 36=AI temperature drop, 37=Time extension, 45=EXT defrost'], \
        ['DI9 TYPE', 191, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital input 3 function, 0=No function, 1=Away, 2=Long away, 3=Overpressure, 4=Cooker hood, 5=Central vacuum cleaner, 6=Max heating, 7=Max cooling, 10=Boosting, 26=EH PDA, 27=Cooling error, 28=Cooling run indication, 29=HR error, 30=HR run indicaion, 34=External heating disable, 35=External cooling disable, 36=AI temperature drop, 37=Time extension, 45=EXT defrost'], \
        ['DO2 TYPE', 192, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital output 2 function, 0=No function, 1=CO2 sensor 1 comparing, 2=CO2 sensor 2 comparing, 3=CO2 sensor 3 comparing, 4=RH sensor 1 comparing, 5=RH sensor 2 comparing, 6=RH sensor 3 comparing, 7=External temp ensor comparing, 8=Room temp 1 comparing, 9=Room temp 2 comparing, 10=Room temp 3 comparing, 11=Temp setpoint, 12=Time relay, 13=External heating disable, 14=External cooling disable'], \
        ['DO3 TYPE', 193, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital output 3 function, 0=No function, 1=CO2 sensor 1 comparing, 2=CO2 sensor 2 comparing, 3=CO2 sensor 3 comparing, 4=RH sensor 1 comparing, 5=RH sensor 2 comparing, 6=RH sensor 3 comparing, 7=External temp ensor comparing, 8=Room temp 1 comparing, 9=Room temp 2 comparing, 10=Room temp 3 comparing, 11=Temp setpoint, 12=Time relay, 13=External heating disable, 14=External cooling disable'], \
        ['DO4 TYPE', 194, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital output 4 function, 0=No function, 1=CO2 sensor 1 comparing, 2=CO2 sensor 2 comparing, 3=CO2 sensor 3 comparing, 4=RH sensor 1 comparing, 5=RH sensor 2 comparing, 6=RH sensor 3 comparing, 7=External temp ensor comparing, 8=Room temp 1 comparing, 9=Room temp 2 comparing, 10=Room temp 3 comparing, 11=Temp setpoint, 12=Time relay, 13=External heating disable, 14=External cooling disable'], \
        ['DO5 TYPE', 195, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital output 5 function, 0=No function, 1=CO2 sensor 1 comparing, 2=CO2 sensor 2 comparing, 3=CO2 sensor 3 comparing, 4=RH sensor 1 comparing, 5=RH sensor 2 comparing, 6=RH sensor 3 comparing, 7=External temp ensor comparing, 8=Room temp 1 comparing, 9=Room temp 2 comparing, 10=Room temp 3 comparing, 11=Temp setpoint, 12=Time relay, 13=External heating disable, 14=External cooling disable'], \
        ['HEAT BLOCK T', 196, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Block limit of heating, heating blocked when outside temperature is above this value (C) (50=5.0)'], \
        ['AI COMP DO OFF', 197, TYPE_HREG, TYPE_INT16, TYPE_R, '(100%) OFF humidity where relay inactive'], \
        ['REL DEL', 198, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Heat pump units HP & EDX delay for fan speed steps, if AC (register 1x0016=0) delay is the time between each relay step, if DC (register 1x0016=1) delay is the time between each % (sec)'], \
        ['USE METHOD', 199, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Unit use, 1=Home, 2=Office, 3=DDC1, 4=DDC2, 5=DDC3'], \
        ['CHGE', 200, TYPE_HREG, TYPE_INT16, TYPE_R, '-'], \
        ['FAN DATA0C', 201, TYPE_HREG, TYPE_INT16, TYPE_R, '-'], \
        ['FAN DATA1C', 202, TYPE_HREG, TYPE_INT16, TYPE_R, '-'], \
        ['FAN DATA2C', 203, TYPE_HREG, TYPE_INT16, TYPE_R, '-'], \
        ['DATA3C', 204, TYPE_HREG, TYPE_INT16, TYPE_R, '-'], \
        ['DATA4C', 205, TYPE_HREG, TYPE_INT16, TYPE_R, '-'], \
        ['DATA5C', 206, TYPE_HREG, TYPE_INT16, TYPE_R, '-'], \
        ['DATA6C', 207, TYPE_HREG, TYPE_INT16, TYPE_R, '-'], \
        ['DATA7C', 208, TYPE_HREG, TYPE_INT16, TYPE_R, '-'], \
        ['QUICKC', 209, TYPE_HREG, TYPE_INT16, TYPE_R, '-'], \
        ['Week timer program 1/20 Days when running', 210, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value = LeastSignificantBit method, for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 1/20 starting time hours', 211, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 1 time program, starting time, hours'], \
        ['Week timer program 1/20 starting time minutes', 212, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 1 time program, starting time, minutes'], \
        ['Week timer program 1/20 stopping time hours', 213, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 1 time program, stopping time, hours'], \
        ['Week timer program 1/20 stopping time minutes', 214, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 1 time program, stopping time, minutes'], \
        ['Week timer program 1/20 Function', 215, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 1 time program function, 0=No selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 2/20 Days when running', 216, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value = LeastSignificantBit method, for example. Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 2/20 starting time hours', 217, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 2 time program, starting time, hours'], \
        ['Week timer program 2/20 starting time minutes', 218, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 2 time program, starting time, minutes'], \
        ['Week timer program 2/20 stopping time hours', 219, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 2 time program, stopping time, hours'], \
        ['Week timer program 2/20 stopping time minutes', 220, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 2 time program, stopping time, minutes'], \
        ['Week timer program 2/20 Function', 221, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 2 time program function, 0=no selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 3/20 Days when running', 222, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value= LeastSignificantBit method, for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 3/20 starting time hours', 223, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 3 time program, starting time, hours'], \
        ['Week timer program 3/20 starting time minutes', 224, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 3 time program, starting time, minutes'], \
        ['Week timer program 3/20 stopping time hours', 225, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 3 time program, stopping time, hours'], \
        ['Week timer program 3/20 stopping time minutes', 226, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 3 time program, stopping time, minutes'], \
        ['Week timer program 3/20 Function', 227, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 3 time program function, 0=no selection, 1=away function, 2= long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 4/20 Days when running', 228, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value= LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 4/20 starting time hours', 229, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 4 time program, starting time, hours'], \
        ['Week timer program 4/20 starting time minutes', 230, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 4 time program, starting time, minutes'], \
        ['Week timer program 4/20 stopping time hours', 231, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 4 time program, stopping time, hours'], \
        ['Week timer program 4/20 stopping time minutes', 232, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 4 time program, stopping time, minutes'], \
        ['Week timer program 4/20 Function', 233, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 4 time program function, 0=no selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 5/20 Days when running', 234, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 5/20 starting time hours', 235, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 5 time program, starting time, hours'], \
        ['Week timer program 5/20 starting time minutes', 236, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 5 time program, starting time, minutes'], \
        ['Week timer program 5/20 stopping time hours', 237, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 5 time program, stopping time, hours'], \
        ['Week timer program 5/20 stopping time minutes', 238, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 5 time program, stopping time, minutes'], \
        ['Week timer program 5/20 Function', 239, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 5 time program function, 0=no selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 6/20 Days when running', 240, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value= LeastSignificantBit method for example Sun is1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 6/20 starting time hours', 241, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 6 time program, starting time, hours'], \
        ['Week timer program 6/20 starting time minutes', 242, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 6 time program, starting time, minutes'], \
        ['Week timer program 6/20 stopping time hours', 243, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 6 time program, stopping time, hours'], \
        ['Week timer program 6/20 stopping time minutes', 244, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 6 time program,stopping time, minutes'], \
        ['Week timer program 6/20 Function', 245, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 6 time program function, 0=no selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 7/20 Days when running', 246, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 7/20 starting time hours', 247, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 7 time program, starting time, hours'], \
        ['Week timer program 7/20 starting time minutes', 248, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 7 time program, starting time, minutes'], \
        ['Week timer program 7/20 stopping time hours', 249, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 7 time program, stopping time, hours'], \
        ['Week timer program 7/20 stopping time minutes', 250, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 7 time program, stopping time, minutes'], \
        ['Week timer program 7/20 Function', 251, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 7 time program function, 0=no selection, 1= away function, 2= lonbg away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 8/20 Days when running', 252, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order of days Sun - Sat. Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 8/20 starting time hours', 253, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 8 time program, starting time, hours'], \
        ['Week timer program 8/20 starting time minutes', 254, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 8 time program, starting time, minutes'], \
        ['Week timer program 8/20 stopping time hours', 255, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 8 time program, stopping time, hours'], \
        ['Week timer program 8/20 stopping time minutes', 256, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 8 time program, stopping time, minutes'], \
        ['Week timer program 8/20 Function', 257, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 8 time program function, 0=no selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 9/20 Days when running', 258, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 9/20 starting time hours', 259, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 9 time program, starting time, hours'], \
        ['Week timer program 9/20 starting time minutes', 260, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 9 time program, starting time, minutes'], \
        ['Week timer program 9/20 stopping time hours', 261, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 9 time program, stopping time, hours'], \
        ['Week timer program 9/20 stopping time minutes', 262, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 9 time program, stopping time, minutes'], \
        ['Week timer program 9/20 Function', 263, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 9 time program function, 0=no selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 10/20 Days when running', 264, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 10/20 starting time hours', 265, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 10 time program, starting time, hours'], \
        ['Week timer program 10/20 starting time minutes', 266, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 10 time program, starting time, minutes'], \
        ['Week timer program 10/20 stopping time hours', 267, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 10 time program, stopping time, hours'], \
        ['Week timer program 10/20 stopping time minutes', 268, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 10 time program, stopping time, minutes'], \
        ['Week timer program 10/20 Function', 269, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 10 time program function, 0=no selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 11/20 Days when running', 270, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 11/20 starting time hours', 271, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 11 time program, starting time, hours'], \
        ['Week timer program 11/20 starting time minutes', 272, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 11 time program, starting time, minutes'], \
        ['Week timer program 11/20 stopping time, hours', 273, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 11 time program, stopping time, hours'], \
        ['Week timer program 11/20 stopping time minutes', 274, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 11 time program, stopping time, minutes'], \
        ['Week timer program 11/20 Function', 275, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 11 time program function, 0=no selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 12/20 Days when running', 276, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 12/20 starting time, hours', 277, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 12 time program, starting time, hours'], \
        ['Week timer program 12/20 starting time, minutes', 278, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 12 time program, starting time, minutes'], \
        ['Week timer program 12/20 stopping time, hours', 279, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 12 time program, stooping time, hours'], \
        ['Week timer program 12/20 stopping time, minutes', 280, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 12 time program, stopping time, minutes'], \
        ['Week timer program 12/20 Function', 281, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 12 time program function, 0=no selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 13/20 Days when running', 282, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are127 = 1111111'], \
        ['Week timer program 13/20 starting time hours', 283, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 13 time program, starting time, hours'], \
        ['Week timer program 13/20 starting time minutes', 284, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 13 time program, starting time, minutes'], \
        ['Week timer program 13/20 stopping time hours', 285, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 13 time program, stopping time, hours'], \
        ['Week timer program 13/20 stopping time minutes', 286, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 13 time program, stopping time, minutes'], \
        ['Week timer program 13/20 Function', 287, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 13 time program function, 0=no selection, 1=away function, 2=long away function, 3=Heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 14/20 Days when running', 288, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value = LeastSignificantBit method for example. Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 14/20 starting time, hours', 289, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 14 time program, starting time, hours'], \
        ['Week timer program 14/20 starting time, minutes', 290, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 14 time program, starting time, minutes'], \
        ['Week timer program 14/20 stopping time, hours', 291, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 14 time program, stopping time, hours'], \
        ['Week timer program 14/20 stopping time, minutes', 292, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 14 time program, stopping time, minutes'], \
        ['Week timer program 14/20 Function', 293, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 14 time program function, 0=no selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 15/20 Days when running', 294, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 15/20 Starting time, hours', 295, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 15 time program, starting time, hours'], \
        ['Week timer program 15/20 Starting time, minutes', 296, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 15 time program, starting time, minutes'], \
        ['Week timer program 15/20 Stopping time, hours', 297, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 15 time program, stopping time, hours'], \
        ['Week timer program 15/20 Stopping time, minutes', 298, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 15 time program, stopping time, minutes'], \
        ['Week timer program 15/20 Function', 299, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 15 time program function, 0=no selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 16/20 Days when running', 300, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat. Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 16/20 Starting time, hours', 301, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 16 time program, starting time, hours'], \
        ['Week timer program 16/20 Starting time, minutes', 302, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 16 time program, starting time, minutes'], \
        ['Week timer program 16/20 Stopping time, hours', 303, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 16 time program, stopping time, hours'], \
        ['Week timer program 16/20 Stopping time, minutes', 304, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 16 time program, stopping time, minutes'], \
        ['Week timer program 16/20 Function', 305, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 16 time program function, 0=no selection, 1=away function, 2=long away function, 3= heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 17/20 Days when running', 306, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 17/20 starting time hours', 307, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 17 time program, starting time, hours'], \
        ['Week timer program 17/20 starting time minutes', 308, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 17 time program, starting time, minutes'], \
        ['Week timer program 17/20 stopping time hours', 309, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 17 time program, stopping time, hours'], \
        ['Week timer program 17/20 stopping time minutes', 310, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 17 time program, stopping time, minutes'], \
        ['Week timer program 17/20 Function', 311, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 17 time program function, 0=no selection, 1=away function, 2=long away function, 3=heating blocked, 4= cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 18/20 Days when running', 312, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 18/20 starting time hours', 313, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 18 time program, starting time, hours'], \
        ['Week timer program 18/20 starting time minutes', 314, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 18 time program, starting time, minutes'], \
        ['Week timer program 18/20 stopping time hours', 315, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 18 time program, stopping time, hours'], \
        ['Week timer program 18/20 stopping time minutes', 316, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 18 time program, stopping time, minutes'], \
        ['Week timer program 18/20 Function', 317, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 18 time program function, 0=no selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature block, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 19/20 Days when running', 318, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 19/20 starting time hours', 319, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 19 time program, starting time, hours'], \
        ['Week timer program 19/20 starting time minutes', 320, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 19 time program, starting time, minutes'], \
        ['Week timer program 19/20 stopping time hours', 321, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 19 time programm, stopping time, hours'], \
        ['Week timer program 19/20 stopping time minutes', 322, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 19 time program, stopping time, minutes'], \
        ['Week timer program 19/20 Function', 323, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 19 time program function, 0=no selection, 1=away function, 2=long away function, 3= heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Week timer program 20/20 Days when running', 324, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Day order Sun - Sat Register value = LeastSignificantBit method for example Sun is 1 = 1000000, Tue is 4 = 0010000 and all days are 127 = 1111111'], \
        ['Week timer program 20/20 starting time hours', 325, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 20 time program, starting time, hours'], \
        ['Week timer program 20/20 starting time minutes', 326, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 20 time program, starting time, minutes'], \
        ['Week timer program 20/20 stopping time hours', 327, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 20 time program, stopping time, hours'], \
        ['Week timer program 20/20 stopping time minutes', 328, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 20 time program, stopping time, minutes'], \
        ['Week timer program 20/20 Function', 329, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Week timer 20 time program function, 0=no selection, 1=away function, 2=long away function, 3= heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Year timer program 1/5 Starts day', 330, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 1 start date'], \
        ['Year timer program 1/5 Starts month', 331, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 1 start month'], \
        ['Year timer program 1/5 Starts year', 332, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 1 start year, Year = Register value + 2000'], \
        ['Year timer program 1/5 Starts hours', 333, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 1 start hours'], \
        ['Year timer program 1/5 Starts minutes', 334, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 1 start minutes'], \
        ['Year timer program 1/5 stops day', 335, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 1 stop date'], \
        ['Year timer program 1/5 stops month', 336, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 1 stop month'], \
        ['Year timer program 1/5 stops year', 337, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 1 stop year, Year = Register value + 2000'], \
        ['Year timer program 1/5 stops hours', 338, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 1 stop hours'], \
        ['Year timer program 1/5 stops minutes', 339, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 1 stop minutes'], \
        ['Year timer program 1/5 Function', 340, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer time program function, 0=No selection, 1=away function, 2=long away function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Year timer program 2/5 Starts day', 341, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 2 start date'], \
        ['Year timer program 2/5 Starts month', 342, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 2 start month'], \
        ['Year timer program 2/5 Starts year', 343, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 2 start yeari, Year = Register value + 2000'], \
        ['Year timer program 2/5 Starts hours', 344, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 2 start hours'], \
        ['Year timer program 2/5 Starts minutes', 345, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 2 start minutes'], \
        ['Year timer program 2/5 Stops day', 346, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 2 stop date'], \
        ['Year timer program 2/5 Stops month', 347, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 2 stop month'], \
        ['Year timer program 2/5 Stops year', 348, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 2 stop year, Year = Register value + 2000'], \
        ['Year timer program 2/5 Stops hours', 349, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 2 stop hours'], \
        ['Year timer program 2/5 Stops minutes', 350, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 2 stop minutes'], \
        ['Year timer program 2/5 Function', 351, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer time program function, 0=no selection, 1=away function, 2=long away poissa function, 3=heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Year timer program 3/5 Starts day', 352, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 3 start date'], \
        ['Year timer program 3/5 Starts month', 353, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 3 start month'], \
        ['Year timer program 3/5 Starts year', 354, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 3 start year, Year = Register value + 2000'], \
        ['Year timer program 3/5 Starts hours', 355, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 3 start hours'], \
        ['Year timer program 3/5 Starts minutes', 356, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 3 start minutes'], \
        ['Year timer program 3/5 Stops day', 357, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 3 stop date'], \
        ['Year timer program 3/5 Stops month', 358, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 3 stop month'], \
        ['Year timer program 3/5 Stops year', 359, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 3 stop year, Year = Register value + 2000'], \
        ['Year timer program 3/5 Stops hours', 360, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 3 stop hours'], \
        ['Year timer program 3/5 Stops minutes', 361, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 3 stop minutes'], \
        ['Year timer program 3/5 Function', 362, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer time program function, 0=no selection, 1=away function, 2=long away function, 3= Heating blocked, 4=cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Year timer program 4/5 Starts day', 363, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 4 start date'], \
        ['Year timer program 4/5 Starts month', 364, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 4 start month'], \
        ['Year timer program 4/5 Starts year', 365, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 4 start year, year = Register value + 2000'], \
        ['Year timer program 4/5 Starts hours', 366, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 4 start hours'], \
        ['Year timer program 4/5 Starts minutes', 367, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 4 start minutes'], \
        ['Year timer program 4/5 Stops day', 368, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 4 stop date'], \
        ['Year timer program 4/5 Stops month', 369, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 4 stop month'], \
        ['Year timer program 4/5 Stops year', 370, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 4 stop year, year = Register value + 2000'], \
        ['Year timer program 4/5 Stops hours', 371, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 4 stop hours'], \
        ['Year timer program 4/5 Stops minutes', 372, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 4 stop minutes'], \
        ['Year timer program 4/5 Function', 373, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer time program function, 0=no selection, 1=away function, 2=long away function, 3=Heating blocked, 4=Cooling blocked, 5=Temperature block 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['Year timer program 5/5 Starts day', 374, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 5 start date'], \
        ['Year timer program 5/5 Starts month', 375, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 5 start month'], \
        ['Year timer program 5/5 Starts year', 376, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 5 start year, year = Register value + 2000'], \
        ['Year timer program 5/5 Starts hours', 377, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 5 start hours'], \
        ['Year timer program 5/5 Starts minutes', 378, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 5 start minutes'], \
        ['Year timer program 5/5 Stops day', 379, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 5 stop date'], \
        ['Year timer program 5/5 Stops month', 380, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 5 stop month'], \
        ['Year timer program 5/5 Stops year', 381, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 5 stop year, year = Register value + 2000'], \
        ['Year timer program 5/5 Stops hours', 382, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 5 stop hours'], \
        ['Year timer program 5/5 Stops minutes', 383, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer 5 stop minutes'], \
        ['Year timer program 5/5 Function', 384, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year timer time program function, 0=no selection, 1=away function, 2=long away function, 3=Heating blocked, 4=Cooling blocked, 5=temperature drop, 6=Max heating, 7=Max cooling, 8-15=AC fan speed 1-8, 16=DO time relay, 20-100=EC fan speed 20-100%'], \
        ['ALARM1 NO', 385, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM1 TYPE', 386, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM1 YY', 387, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM1 MM', 388, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM1 DD', 389, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM1 HH', 390, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM1 MI', 391, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM2 NO', 392, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM2 TYPE', 393, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM2 YY', 394, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM2 MM', 395, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM2 DD', 396, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM2 HH', 397, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM2 MI', 398, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM3 NO', 399, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM3 TYPE', 400, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM3 YY', 401, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM3 MM', 402, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM3 DD', 403, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM3 HH', 404, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM3 MI', 405, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM4 NO', 406, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM4 TYPE', 407, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM4 YY', 408, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM4 MM', 409, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM4 DD', 410, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM4 HH', 411, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM4 MI', 412, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM5 NO', 413, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM5 TYPE', 414, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM5 YY', 415, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM5 MM', 416, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM5 DD', 417, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM5 HH', 418, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM5 MI', 419, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM6 NO', 420, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM6 TYPE', 421, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM6 YY', 422, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM6 MM', 423, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM6 DD', 424, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM6 HH', 425, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM6 MI', 426, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM7 NO', 427, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM7 TYPE', 428, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM7 YY', 429, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM7 MM', 430, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM7 DD', 431, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM7 HH', 432, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM7 MI', 433, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM8 NO', 434, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM8 TYPE', 435, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM8 YY', 436, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM8 MM', 437, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM8 DD', 438, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM8 HH', 439, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM8 MI', 440, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM9 NO', 441, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM9 TYPE', 442, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM9 YY', 443, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM9 MM', 444, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM9 DD', 445, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM9 HH', 446, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM9 MI', 447, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM10 NO', 448, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM10 TYPE', 449, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM10 YY', 450, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM10 MM', 451, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM10 DD', 452, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM10 HH', 453, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM10 MI', 454, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM11 NO', 455, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM11 TYPE', 456, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM11 YY', 457, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM11 MM', 458, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM11 DD', 459, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM11 HH', 460, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM11 MI', 461, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM12 NO', 462, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM12 TYPE', 463, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM12 YY', 464, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM12 MM', 465, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM12 DD', 466, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM12 HH', 467, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM12 MI', 468, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM13 NO', 469, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fanpressure error, 21=Extract fan pressure error'], \
        ['ALARM13 TYPE', 470, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM13 YY', 471, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM13 MM', 472, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM13 DD', 473, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM13 HH', 474, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM13 MI', 475, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM14 NO', 476, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM14 TYPE', 477, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM14 YY', 478, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM14 MM', 479, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM14 DD', 480, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM14 HH', 481, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM14 MI', 482, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM15 NO', 483, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM15 TYPE', 484, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM15 YY', 485, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM15 MM', 486, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM15 DD', 487, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM15 HH', 488, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM15 MI', 489, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM16 NO', 490, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM16 TYPE', 491, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM16 YY', 492, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM16 MM', 493, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM16 DD', 494, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM16 HH', 495, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM16 MI', 496, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM17 NO', 497, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM17 TYPE', 498, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM17 YY', 499, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM17 MM', 500, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM17 DD', 501, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM17 HH', 502, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM17 MI', 503, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM18 NO', 504, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Roomtemp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM18 TYPE', 505, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM18 YY', 506, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM18 MM', 507, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM18 DD', 508, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM18 HH', 509, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM18 MI', 510, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM19 NO', 511, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM19 TYPE', 512, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM19 YY', 513, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM19 MM', 514, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM19 DD', 515, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM19 HH', 516, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM19 MI', 517, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM20 NO', 518, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm number, 1=TE5 Supply air after HR cold, 2=TE10 Supply air after heater cold, 3=TE10 Supply air after heater hot, 4=TE20 Room temp hot, 5=TE30 Extract air cold, 6=TE30 Extract air hot, 7=HP error, 8=EH error, 9=Return water cold, 10=HR error, 11=Cooling error, 12=Emergency stop, 13=Fire risk, 14=Service reminder, 15=EH PDA, 16=Supply filter dirty, 17=Extract filter dirty, 20=Supply fan pressure error, 21=Extract fan pressure error'], \
        ['ALARM20 TYPE', 519, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm state, 0=OFF, 1=RESET, 2=ON'], \
        ['ALARM20 YY', 520, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time year, Year = register value + 2000'], \
        ['ALARM20 MM', 521, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time month'], \
        ['ALARM20 DD', 522, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time day'], \
        ['ALARM20 HH', 523, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time hour'], \
        ['ALARM20 MI', 524, TYPE_HREG, TYPE_INT16, TYPE_R, 'Alarm time minutes'], \
        ['ALARM D TE5 L', 525, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay supply air after HR cold (sec)'], \
        ['ALARM D TE10 L', 526, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay supply air after heater cold (sec)'], \
        ['ALARM D TE10 H', 527, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay supply air after heater hot (sec)'], \
        ['ALARM D TE20 H', 528, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay room air hot (sec)'], \
        ['ALARM D TE30 L', 529, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay extract air cold (sec)'], \
        ['ALARM D TE30 H', 530, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay extract air hot (sec)'], \
        ['ALARM D HP', 531, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay HP compressor error (sec)'], \
        ['ALARM D SLP', 532, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay electrical heater overheated (sec)'], \
        ['ALARM D TE45 L', 533, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay water heater return water freeze risk (sec)'], \
        ['ALARM D LTO', 534, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay HR error (sec)'], \
        ['ALARM D COOL', 535, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay cool error (sec)'], \
        ['ALARM D EMERGENCY', 536, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay external emergency stop (sec)'], \
        ['ALARM D FIRE', 537, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay fire risk (sec)'], \
        ['ALARM D SERVICE', 538, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay service reminder (vrk)'], \
        ['ALARM D PDS10', 539, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay PDS10 electrical heater guard (sec)'], \
        ['ALARM D SPLY FILT H', 540, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay supply filter pressure guard (sec)'], \
        ['ALARM D EXT FILT H', 541, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay extract filter pressure guard (sec)'], \
        ['ALARM SPLY FILT L', 542, TYPE_HREG, TYPE_INT16, TYPE_R, 'Not in use'], \
        ['ALARM EXT FILT L', 543, TYPE_HREG, TYPE_INT16, TYPE_R, 'Not in use'], \
        ['ALARM TF PA', 544, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay constant duct pressure deviation alarm supply (sec)'], \
        ['ALARM PF PA', 545, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm delay constant duct pressure deviation alarm extract (sec)'], \
        ['ALARM TE5 L', 551, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Alarm limit supply air after HR cold (C) (50=5.0)'], \
        ['ALARM TE10 L', 552, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Alarm limit supply air after heater cold (C) (50=5.0)'], \
        ['ALARM TE10 H', 553, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Alarm limit supply air after heater hot (C) (50=5.0)'], \
        ['ALARM TE20 H', 554, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Alarm limit room air hot (C) (50=5.0)'], \
        ['ALARM TE30 L', 555, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Alarm limit extract air cold (C) (50=5.0)'], \
        ['ALARM TE30 H', 556, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Alarm limit extract air hot (C) (50=5.0)'], \
        ['ALARM HP', 557, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm limit HP compressor error (not in use)'], \
        ['ALARM SLP', 558, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm limit electrical heater overheated (not in use)'], \
        ['ALARM TE45 L', 559, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Alarm limit water heater return water freeze risk (C) (50=5.0)'], \
        ['ALARM SPLY FILT H', 566, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm limit supply filter pressure guard'], \
        ['ALARM EXT FILT H', 567, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Alarm limit extract filter pressure guard'], \
        ['ALARM TE10 L A', 572, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Supply air after heater A alarm limit from RESET mode NB! Has to be same as #552'], \
        ['ALARM HYSTER', 577, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Hysteresis for alarms (C) (50=5.0)'], \
        ['B ALARM START', 578, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Time when B alarms can give an external indication, start time ex 9 (hours)'], \
        ['B ALARM STOP', 579, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Time when B alarms can give an external indication, stop time ex 16 (hours)'], \
        ['B ALARM WEEK', 580, TYPE_HREG, TYPE_UINT16, TYPE_RW, 'Weekdays when B alarms can give an external indication, Register value = 65408 + days chosen sun=1 mon=2, tue=4, wed=8, thu=16, fri=32, sat=64 and combinations sum up'], \
        ['N O ALARMS', 581, TYPE_HREG, TYPE_UINT16, TYPE_R, 'Number of alarms'], \
        ['C MIN RTC', 582, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Time setting that updates RTC circuit (min)'], \
        ['C HOUR RTC', 583, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Time setting that updates RTC circuit (hours)'], \
        ['C DAY RTC', 584, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Day setting that updates RTC circuit'], \
        ['C MONTH RTC', 585, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Month setting that updates RTC circuit'], \
        ['C YEAR RTC', 586, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Year setting that updates RTC circuit, Year = register value + 2000'], \
        ['C WEEK RTC', 587, TYPE_HREG, TYPE_INT16, TYPE_R, 'Weekday, Mon = 1, Tue = 2, ... Sun = 0, caluculates itself'], \
        ['REAL SP', 591, TYPE_HREG, TYPE_INT16_10, TYPE_R, 'Real setpoint on main display if for example Temperature drop is active (C) (50=5.0)'], \
        ['GAIN HumEXT', 592, TYPE_HREG, TYPE_INT16, TYPE_R, 'Extract air humidity factor Fixed!'], \
        ['GAIN T EXT', 593, TYPE_HREG, TYPE_INT16_1000, TYPE_R, 'Extract air temperature factor *0.001 Fixed!'], \
        ['SLP PWM CYCLE', 594, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Electrical heater PWM cycle length (sec)'], \
        ['FREE DO', 595, TYPE_HREG, TYPE_INT16, TYPE_R, 'Number of free DO:s'], \
        ['PRO SIZE', 596, TYPE_HREG, TYPE_INT16, TYPE_RW, 'PRO units unit size (register 1x0051=1), (register 3x0597=2,3,4 or 5) 0=10, 1=20, 2=25, 3=35, 4=50, 5=70, 6=90, 7=120, 8=150, 9=180, (register 3x0597=0 or 1) 0=12, 1=20, 2=25'], \
        ['FAMILY TYPE', 597, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Unit type Family (register 1x0051=0), 0=Pingvin, 1=Pandion, 2=Pelican, 3=Pegasos, 4=Pegasos XL, 5=LTR-3, 6=LTR-6, 7=LTR-7, 8=LTR-7 XL // PRO (register 1x0051=1), 0=RS, 1=RSC, 2=LTR, 3=LTC, 4=LTT, 5=LTP'], \
        ['SN', 598, TYPE_HREG, TYPE_INT16, TYPE_R, 'Serial number given during testing at factory'], \
        ['SW', 599, TYPE_HREG, TYPE_INT16, TYPE_R, 'Programme version Fixed!'], \
        ['SPLY Dz', 600, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Supply air controller dead zone (C) (50=5.0)'], \
        ['VPK Integration time', 601, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Return water controller integration time in Stop mode (sec)'], \
        ['VPK Reset time', 602, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Return water controller reset time in Stop mode (sec)'], \
        ['VPK Dz', 603, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Return water controller dead zone in Stop mode (C) (50=5.0)'], \
        ['VPK DELAY', 604, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Return water pump is always running when, outside air temp drops below +10 C or when supply air controller is at 100% or more. When control drops below 100% or to HR stage the return water pump has got a delay so that it is running although the valve is fully closed.'], \
        ['AO3 LOW', 605, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Cooling stage AO output voltage limitation, lowest voltage (%) 0-100% = 0-10VDC'], \
        ['AO3 HIGH', 606, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Cooling stage AO output voltage limitation, highest voltage (%) 0-100% = 0-10VDC'], \
        ['AO5 LOW', 607, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Heating stage AO output voltage limitation, lowest voltage (%) 0-100% = 0-10VDC'], \
        ['AO5 HIGH', 608, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Heating stage AO output voltage limitation, highest voltage (%) 0-100% = 0-10VDC'], \
        ['DO2 ON', 609, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital output 2 limitvalue for on switching if for ex. register 3x0192=4'], \
        ['DO2 OFF', 610, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital output 2 limitvalue for off switching if for ex. register 3x0192=4'], \
        ['DO3 ON', 611, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital output 3 limitvalue for on switching if for ex. register 3x0192=4'], \
        ['DO3 OFF', 612, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital output 3 limitvalue for off switching if for ex. register 3x0192=4'], \
        ['DO4 ON', 613, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital output 4 limitvalue for on switching if for ex. register 3x0192=4'], \
        ['DO4 OFF', 614, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital output 4 limitvalue for off switching if for ex. register 3x0192=4'], \
        ['DO5 ON', 615, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital output 5 limitvalue for on switching if for ex. register 3x0192=4'], \
        ['DO5 OFF', 616, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Digital output 5 limitvalue for off switching if for ex. register 3x0192=4'], \
        ['Pres SPLYF VL', 617, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Supply filter pressure switch, low voltage level (V) (100=10.0VDC)'], \
        ['Pres SPLYF VH', 618, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Supply filter pressure switch, high voltage level (V) (100=10.0VDC)'], \
        ['Pres SPLYF RL', 619, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Supply filter pressure switch, corresponding measurement of low voltage level (Pa)'], \
        ['Pres SPLYF RH', 620, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Supply filter pressure switch, corresponding measurement of high voltage level (Pa)'], \
        ['Pres EXTF VL', 621, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Exhaust filter pressure switch, low voltage level (V) (100=10.0VDC)'], \
        ['Pres EXTF VH', 622, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Exhaust filter pressure switch, high voltage level (V) (100=10.0VDC)'], \
        ['Pres EXTF RL', 623, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Exhaust filter pressure switch, corresponding measurement of low voltage level Poistos (Pa)'], \
        ['Pres EXTF RH', 624, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Exhaust filter pressure switch, corresponding measurement of high voltage level Poistos (Pa)'], \
        ['Pres LTO VL', 625, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Heat recovery pressure switch, low voltage level (V) (100=10.0VDC)'], \
        ['Pres LTO VH', 626, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Heat recovery pressure switch, high voltage level (V) (100=10.0VDC)'], \
        ['Pres LTO RL', 627, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Heat recovery pressure switch, corresponding measurement of low voltage (Pa)'], \
        ['Pres LTO RH', 628, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Heat recovery pressure switch, corresponding measurement of high voltage (Pa)'], \
        ['Pres SPLYF RES', 629, TYPE_HREG, TYPE_INT16, TYPE_R, 'Supply filter calculated result pressure (Pa)'], \
        ['Pres EXTF RES', 630, TYPE_HREG, TYPE_INT16, TYPE_R, 'Exhaust filter calculated result pressure (Pa)'], \
        ['Pres LTO RES', 631, TYPE_HREG, TYPE_INT16, TYPE_R, 'Heat recovery calculated result pressure (Pa)'], \
        ['PA ALARM', 632, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Constant duct pressure control, deviation pressure which causes an alarm (Pa)'], \
        ['VKP SPLY MAX', 633, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Constant duct pressure control supply side max pressure (Pa)'], \
        ['VKP EXT MAX', 634, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Constant duct pressure control exhaust side max pressure (Pa)'], \
        ['VKP SPLY MIN', 635, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Constant duct pressure control supply side min pressure (Pa)'], \
        ['VKP EXT MIN', 636, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Constant duct pressure control exhaust side min pressure (Pa)'], \
        ['VKP SPLY RES', 637, TYPE_HREG, TYPE_INT16, TYPE_R, 'Constant duct pressure control supply side calculated pressure (Pa)'], \
        ['VKP EXT RES', 638, TYPE_HREG, TYPE_INT16, TYPE_R, 'Constant duct pressure control exhaust side calculated pressure (Pa)'], \
        ['EXT HP', 639, TYPE_HREG, TYPE_INT16, TYPE_RW, 'EDX, HP units with outdoor pump unit, In use 1 / Not in use 0'], \
        ['Modbus address', 640, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Units ModBus address'], \
        ['PRO Out temp max restriction of capacity', 641, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'PRO unit (register 1x0051=1) outside temp, when unit is allowed to run on max capacity (C) (50=5.0)'], \
        ['PRO Out temp min restriction of capacity', 642, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'PRO unit (register 1x0051=1) outside temp, when unit is restricted to run on min capacity (C) (50=5.0)'], \
        ['LTO FORCE T', 643, TYPE_HREG, TYPE_INT16_10, TYPE_RW, 'Heat recovery, winter forced control temperature limit. When outside temp. drops below this value, heat recovery will be forced on and run on max capacity'], \
        ['HP LTO DEFROST DELAY', 644, TYPE_HREG, TYPE_INT16, TYPE_RW, 'HP unit delay after defrost where heat pump is off for defrosting of HR and extract air fan (min)'], \
        ['EC P value', 645, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Constant duct pressure control proportional band with EC fans (Pa)'], \
        ['VKPS EC Integration time', 646, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Constant duct pressure control integration time with EC fans (sec)'], \
        ['VKPS EC Reset time', 647, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Constant duct pressure control reset time with EC fans (sec)'], \
        ['VKPS EC Dz', 648, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Constant duct pressure dead zone with EC fans (Pa)'], \
        ['VKPS AC t', 649, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Constant duct pressure speed change delay with AC fans (sec)'], \
        ['VKPS AC Dz', 650, TYPE_HREG, TYPE_INT16, TYPE_RW, 'Constant duct pressure dead zone with AC fans (Pa)'], \
        ['CX fan speed', 651, TYPE_HREG, TYPE_INT16, TYPE_RW, 'CX unit fan speed during cooling'] \
    ] }



###########################################################################

# Classes


class EdaModbusRTUListener(log.Logging, modbus.ModbusDataListener):
    def __init__(self, serialPort):
        log.Logging.__init__(self, 'EdaModbusRTUListener')
        
        self.serialPort = serialPort
        self.modbusInterface = modbus.ModbusSlaveSerialRTUModbusListener(self.serialPort)
        self.modbusInterface.setModbusDataListener(self)
        self.modbusInterface.setBaudrate(BAUDRATE)
        self.modbusInterface.setParity('NONE')
        self.modbusInterface.setDataBits('8')
        self.modbusInterface.setStopBits('1')

        self.offbyonefix = False
        self.coils = {}
        self.hregs = {} 
        self.data_lock = threads.Lock()

    def setOffByOneFix(self, enabled):
        self.offbyonefix = enabled

    def init(self):
        if not self.modbusInterface.startListener():
            self.Log("ERROR: Cannot initialize modbus RTU listener for port " + self.serialPort)
            return 0
        return 1
            
    def close(self):
        if self.modbusInterface.isRunning():
            self.modbusInterface.terminate()
            self.modbusInterface.wait_sleeping()

    def dataReceived(self, unitId, coils, hregs):
        self.Log("Received data for unitId %d" % unitId)
        t = time.time()
    
        if self.data_lock.lock_wait():
            if coils:
                for k in coils.keys():
                    self.coils[k] = [t, coils[k]]
            if hregs:
                for k in hregs.keys():
                    self.hregs[k] = [t, hregs[k]]
            self.data_lock.free()
        else:
            self.Log("Unable to get data lock, data lost.")
            
    def runQueryCommand(self, cmd):
        if self.modbusInterface.hasTerminated() and self.modbusInterface.isFailed():
            if not self.init():
                self.Log("Could not restart module after being failed.")
            else:
                self.Log("Module restarted after being failed.")
        
        id = 0
        type = 0
        for t in EDA_DEVICES['DEFAULT']:
            if t[0] == cmd:
                if self.offbyonefix:
                    id = t[1] + 1
                else:
                    id = t[1]
                type = t[2]
                itype = t[3]
                break

        if type == 0:
            self.Log("ERROR: Invalid Enervent EDA key: " + cmd)
            return ""

        res = self.runQueryId(id, type)
        if res != None:
            return convertEnerventMessage(itype, res)
        return ""

    def runQueryId(self, id, type):
        self.Debug("Running enerventEda query with id %d, type %d" % (id, type))

        res = None
        if self.data_lock.lock_wait():
            temp = None
            if type == TYPE_COIL:            
                if self.coils.has_key(id):
                    temp = self.coils[id]
            elif type == TYPE_HREG:
                if self.hregs.has_key(id):
                    temp = self.hregs[id]
                
            if temp != None:
                if (time.time() - temp[0]) <= DATAVALID:
                    res = temp[1]
                else:
                    self.Debug("Data expired with id %d" % id)
            else:
                self.Debug("No data found with id %d" % id)
            self.data_lock.free()
        else:
            self.Log("Cannot get data lock.")
        return res   


###########################################################################

# Functions

def convertEnerventMessage(type, value):
    if type == TYPE_UINT8:
        temp = "%d" % (value & 0x00FF)
        return temp
    elif type == TYPE_UINT16:
        temp = "%d" % value
        return temp
    elif type == TYPE_INT16:
        temp = binary.twosComplementToInt(value, 16)
        temp = "%d" % temp
        return temp
    elif type == TYPE_INT16_10:
        temp = binary.twosComplementToInt(value, 16)
        temp = "%.1f" % (temp / 10.0)
        return temp
    elif type == TYPE_INT16_1000:
        temp = binary.twosComplementToInt(value, 16)
        temp = "%.3f" % (temp / 1000.0)
        return temp

    return ""
