#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            raspberryGpioThread.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.7j
#
# Date:            05.02.2017
#
# Description:     Standalone python process to interact with GPIO.
#                  Should be executed with user id 0 (root).
#                  
# Requirements:    Python interpreter 2.6 or newer (www.python.org)
#
# Version history: ** 28.12.2014 v1.6d (Olli Lammi) **
#                  First version.
#
#                  ** 05.02.2017 v1.7j (Olli Lammi) **
#                  Added counter functionality.
#
###########################################################################

# Commands:
#
#     hello
#     input <channel> [<flag>]   (<flag> = NONE|PULLUP|PULLDOWN)
#     counter <channel> [<flag> [<bouncetime>]] (<flag> = NONE|PULLUP|PULLDOWN)
#                     (<bouncetime> = Input bounce time in milliseconds.)
#     init
#     read <channel>
#     exit

# Imports

import sys, os
import string
import thread

import RPi.GPIO as GPIO

###########################################################################

# Classes

class Lock(object):
    def __init__(self):
        self.mylock = thread.allocate_lock()

    def lock_wait(self):
        return self.mylock.acquire()

    def lock_immediate(self):
        return self.mylock.acquire(0)

    def free(self):
        self.mylock.release()


###########################################################################

# State variables

hasInit = False
channels = {}
counters = {}
lock = Lock()


###########################################################################

# Functions

def doOutput(str):
    sys.stdout.write(str + chr(10))
    sys.stdout.flush()

def startsWith(str, needle):
    if len(needle) <= 0:
        return True
    if len(str) < len(needle):
        return False
    if str[:len(needle)] == needle:
        return True
    return False

def handleCommand(buff):
    if buff == 'exit':
        return False

    if buff == 'hello':
        doOutput('OK')
        return True

    if buff == 'init':
        return handleInit()
    
    if startsWith(buff, 'input'):
        return handleInput(buff[5:])

    if startsWith(buff, 'counter'):
        return handleCounter(buff[7:])

    if startsWith(buff, 'read'):
        return handleRead(buff[4:])

    doOutput('ERROR:Invalid command')
    return True 

def input_edge_callback(channel):
    key = int(channel)
    if lock.lock_wait():
        try:
            if not counters.has_key(key):
                counters[key] = 1
            else:
                counters[key] = counters[key] + 1
        finally:
            lock.free()

def handleInit():
    global hasInit
    if hasInit:
        doOutput('ERROR:Already init')
        return True

    for key in channels.keys():
        type = string.strip(string.upper(channels[key]['type']))
        flag = string.strip(string.upper(channels[key]['flag']))

        if flag == 'NONE':
            GPIO.setup(int(key), GPIO.IN)
        elif flag == 'PULLUP':
            GPIO.setup(int(key), GPIO.IN, pull_up_down=GPIO.PUD_UP)
        elif flag == 'PULLDOWN':
            GPIO.setup(int(key), GPIO.IN, pull_up_down=GPIO.PUD_DOWN)
        else:
            doOutput('ERROR:Invalid flag')
            return False

        if type == 'COUNTER':
            bounce = channels[key]['bounce']
            if bounce > 0:
                GPIO.add_event_detect(int(key), GPIO.BOTH, callback=input_edge_callback, bouncetime=bounce)
            else:
                GPIO.add_event_detect(int(key), GPIO.BOTH, callback=input_edge_callback)

    hasInit = True
    doOutput('OK')
    return True


def handleInput(cmd):
    if hasInit:
        doOutput('ERROR:Already init')
        return True
    temp = string.split(string.strip(cmd), ' ')
    if len(temp) < 1:
        doOutput('ERROR:Invalid input')
        return True

    flag = 'NONE'
    if len(temp) > 1:
        flag = string.upper(string.strip(temp[1]))
    channel = string.strip(temp[0])
    if len(channel) < 1:
        doOutput('ERROR:Invalid input')
        return True
    channels[channel] = {'type': 'INPUT', 'flag': flag}
    doOutput('OK')
    return True


def handleCounter(cmd):
    if hasInit:
        doOutput('ERROR:Already init')
        return True
    temp = string.split(string.strip(cmd), ' ')
    if len(temp) < 1:
        doOutput('ERROR:Invalid input')
        return True

    flag = 'NONE'
    bounce = 0
    if len(temp) > 1:
        flag = string.upper(string.strip(temp[1]))
        if len(temp) > 2:
            try:
                bounce = int(string.strip(temp[2]))
            except:
                doOutput('ERROR:Invalid bounce time.')
                return True
    channel = string.strip(temp[0])    
    if len(channel) < 1:
        doOutput('ERROR:Invalid input')
        return True
    channels[channel] = {'type': 'COUNTER', 'flag': flag, 'bounce': bounce}
    doOutput('OK')
    return True


def handleRead(cmd):
    if not hasInit:
        doOutput('ERROR:Not init')
        return True

    temp = string.split(string.strip(cmd), ' ')
    if len(temp) != 1:
        doOutput('ERROR:Invalid read')
        return True

    channel = string.strip(temp[0])

    if not channels.has_key(channel):
        doOutput('ERROR:No channel configured')
        return True
    
    chandata = channels[channel]

    key = int(channel)
    if chandata['type'] == 'INPUT':
        res = GPIO.input(key)
    elif chandata['type'] == 'COUNTER':
        res = 0

        if lock.lock_wait():
            try:
                if counters.has_key(key):
                    res = counters[key]
                    counters[key] = 0
            finally:
                lock.free()
 
    doOutput("%d" % res)
    return True


###########################################################################

# Main


buff = ""

GPIO.setmode(GPIO.BOARD)

while 1:
    tempc = sys.stdin.read(1)
    if not tempc:
        break
    if tempc == chr(10):
        if not handleCommand(buff):
            break
        buff = ""
    else:
        buff = buff + tempc

if hasInit:        
  GPIO.cleanup()
sys.exit(0)

