#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            telldusCore.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.6c
#
# Date:            01.02.2015
#
# Description:     Library to communicate with wireless sensors and 
#                  Telldus Tellstick Duo using telldus-core-library.
#                  
# Requirements:    Python interpreter 2.5 or newer (www.python.org)
#
#                  For Telldus Live API requires Python OAuth.
#                  ("sudo apt-get install python-oauth" for "debians")
#
# Version history: ** 12.05.2014 v1.6a (Olli Lammi) **
#                  First version.
#
#                  ** 22.01.2015 v1.6b (Olli Lammi) **
#                  Added support for Telldus TellStick Net using
#                  Telldus Live network service API.
#
#                  ** 01.02.2015 v1.6c (Olli Lammi) **
#                  Fixed old LIB type functionality broken after Live
#                  version.
#
###########################################################################

# Imports

import sys, os 
import string, ctypes
import time

import httplib, urllib, json

from modules.core import log
from modules.utils import utils

# oauth module, not loaded until required
oauth = None

###########################################################################

# Constants

SENSOR_DATATYPES = { 'TEMPERATURE': 1, 'HUMIDITY': 2, 'RAINRATE': 4, \
                     'RAINTOTAL': 8, 'WINDDIRECTION': 16, 'WINDAVERAGE': 32, \
                     'WINDGUST': 64 }

SENSOR_LIVE_DATATYPES = { 'TEMP': 'TEMPERATURE', 'HUMIDITY': 'HUMIDITY'}

DATA_LEN = 256

TELLDUS_LIVE_API_NAME = "api.telldus.com"

###########################################################################

# Classes

class TelldusSource(log.Logging):
    def __init__(self):
        log.Logging.__init__(self, 'TelldusSource')
        self.DATAPOINTS = {}
        self.libType = 'LIB'
        self.libtd = None
        
        self.livePairs = None
        self.publicKey = None
        self.privateKey = None
        self.authToken = None
        self.authTokenSecret = None
        
    def initLibrary(self):
        if self.libType == 'LIB':            
            self.libtd = ctypes.cdll.LoadLibrary('libtelldus-core.so.2')
            if self.libtd == None:
                return 0
        
            try:
                self.Debug("Initializing telldus-core library.")
                self.libtd.tdInit()
            except:
                self.Log("Cannot initialize telldus-core library.")
                self.libtd = None
                return 0
    
            self.Log("Initialized telldus-core library.")
            
        elif self.libType == 'LIVE':
            try:
                self.livePairs = {}
                self.Debug("Initializing Telldus LIVE service API library.")
                response = self.liveApiRequest('sensors/list', {'includeIgnored': 0})
                if not response or not response.has_key('sensor'):
                    self.Log("Error initializing Telldus LIVE service API: %s" % response['error'])
                    return 0
                self.Debug("Received list of %d sensors from Telldus Live API" % len(response['sensor']))
                for sensor in response['sensor']:
                    self.Debug("  Live id: %s name: %s" % (sensor['id'], sensor['name']))
                    sensResponse = self.liveApiRequest('sensor/info', {'id': sensor['id'] })

                    try:
                        sensorId = int(sensResponse['sensorId'])
                    except:
                        sensorId = None
                        self.Log("ERROR: Error converting Live Sensor ID to integer: %s", sensResponse['sensorId'])
                        continue
                    
                    for sensData in sensResponse['data']:
                        dataType = string.upper(sensData['name'])
                        if dataType in SENSOR_LIVE_DATATYPES.keys():
                            dataType = SENSOR_LIVE_DATATYPES[dataType]
                        
                        self.Debug("    protocol: %s, id: %d, datatype: %s" % (sensResponse['protocol'], sensorId, dataType))
                    
                        if not dataType in SENSOR_DATATYPES.keys():
                            self.Log("WARNING: Unknown sensor datatype %s" % dataType)

                        for dpkey in self.DATAPOINTS.keys():
                            dp = self.DATAPOINTS[dpkey]
                            if dp[0] == sensResponse['protocol'] and dp[2] == sensorId and dp[3] == dataType:
                                self.livePairs[dpkey] = sensor['id']
                                self.Debug("      paired LIVE sensor id %s with data point %s" % (sensor['id'], dpkey))
                                break
                for dpkey in self.DATAPOINTS.keys():
                    if not self.livePairs.has_key(dpkey):
                        self.Log("Could not pair data point %s with Telldus Live service API sensor." % dpkey)
            except Exception, e:
                self.Log("Cannot initialize Telldus LIVE service API library.")
                self.Log("Exception: %s" % e.__str__())
                return 0
            
        return 1

    def closeLibrary(self):
        if self.libType == 'LIB':        
            try:
                self.Debug("Closing telldus-core library.")
                self.libtd.tdClose()
            except:
                self.Debug("Error closing libtelldus-core.")

            self.libtd = None

    def setLibraryType(self, libt):
        self.libType = libt
    
    def setDatapoints(self, pointmap):
        self.DATAPOINTS = pointmap

    def setLiveAuth(self, pubkey, privkey, token, tokensecret):
        self.publicKey = pubkey
        self.privateKey = privkey
        self.authToken = token
        self.authTokenSecret = tokensecret

    def runQueryCommand(self, cmds):
        if self.libType == 'LIB':
            return self.runQueryCommandLib(cmds)
        elif self.libType == 'LIVE':
            return self.runQueryCommandLive(cmds)
        else:
            self.Log("ERROR: Invalid library type.")
            return {}

    def runQueryCommandLib(self, cmds):
        if self.libtd == None:
            self.Log("ERROR: No libtelldus-core.so.2 library available.")
            return {}
        
        sensordata = []
        protocol = ctypes.create_string_buffer(DATA_LEN)
        model = ctypes.create_string_buffer(DATA_LEN)
        sensorId = ctypes.c_int(0)
        dataTypes = ctypes.c_int(0)
        value = ctypes.create_string_buffer(DATA_LEN)
        tstamp = ctypes.c_int(0)
        
        try:
            while self.libtd.tdSensor(protocol, DATA_LEN, model, DATA_LEN, \
                                      ctypes.byref(sensorId), ctypes.byref(dataTypes)) == 0:
                self.Debug("Found sensor %s:%s:%d" % (protocol.value, model.value, sensorId.value))
    
                for dtype in SENSOR_DATATYPES.keys():
                    dtid = SENSOR_DATATYPES[dtype]
                    if dataTypes.value & dtid:
                        self.Debug("  Reading sensor data type: %s" % dtype)
                        try:
                            self.libtd.tdSensorValue(protocol, model, sensorId, dtid, \
                                                     value, DATA_LEN, ctypes.byref(tstamp))
                            sensordata.append([protocol.value, model.value, sensorId.value, dtid, value.value])
                        except:
                            self.Log("Error reading Telldus sensor value.")
        except:
            self.Debug("Error reading Telldus sensors.")
                
        self.Debug("Got %d sensor values from telldus-core." % len(sensordata))

        if len(sensordata) <= 0:
            self.Log("No sensor data from telldus-core.")
            return {}
        
        dataresult = {}
                
        for cmd in cmds:
            if not self.DATAPOINTS.has_key(cmd):
                self.Log("ERROR: Undefined data point: " + cmd)
                continue
            item = self.DATAPOINTS[cmd]
                                
            protocol = item[0]
            model = item[1]
            id = item[2]
            datatype = item[3]                    

            if datatype not in SENSOR_DATATYPES.keys():                
                self.Log("ERROR: Invalid or missing data type for data point: " + cmd)
                continue
            
            found = 0
            for sensoritem in sensordata:
                if sensoritem[0] == protocol and sensoritem[1] == model and \
                    sensoritem[2] == id and SENSOR_DATATYPES[datatype] == sensoritem[3]:
                    found = 1
                    dataresult[cmd] = sensoritem[4]

            if not found:            
                self.Log("ERROR: Sensor data not found: " + cmd)
                continue
                
        return dataresult

    def runQueryCommandLive(self, cmds):
        retrSensors = []
        isInitted = False
        for cmd in cmds:
            if self.livePairs.has_key(cmd):
                if not self.livePairs[cmd] in retrSensors:
                    retrSensors.append(self.livePairs[cmd])
            elif not isInitted:
                self.Log("Configured datapoint sensor %s not found in the Telldus sensors. Re-initializing Telldus module." % cmd)
                self.initLibrary()
                isInitted = True
                if self.livePairs.has_key(cmd):
                    self.Log("Datapoint sensor found after re-initialization.")
                    if not self.livePairs[cmd] in retrSensors:
                        retrSensors.append(self.livePairs[cmd])
                else:
                    self.Log("ERROR: Datapoint %s not found in Telldus Live service." % cmd)
            else:
                self.Log("ERROR: Datapoint %s not found in Telldus Live service." % cmd)
        
        sensData = {}
        for liveSensorId in retrSensors:
            self.Debug("Retrieving sensor info from Telldus Live service API for sensor id %s" % liveSensorId)
            sensResponse = self.liveApiRequest('sensor/info', {'id': liveSensorId })

            try:
                sensorId = int(sensResponse['sensorId'])
            except:
                sensorId = None
                self.Log("ERROR: Error converting Live Sensor ID to integer: %s", sensResponse['sensorId'])
                continue
            try:
                lastUpd = int(sensResponse['lastUpdated'])
            except:
                lastUpd = None
                self.Log("ERROR: Error converting Live Sensor last updated value integer: %s", sensResponse['lastUpdated'])
            lastUpdStr = ''
            if lastUpd != None:
                lastUpdStr = time.strftime('%d.%m.%Y %H:%M:%S', time.localtime(lastUpd))
            
            for d in sensResponse['data']:                    
                dataType = string.upper(d['name'])
                if dataType in SENSOR_LIVE_DATATYPES.keys():
                    dataType = SENSOR_LIVE_DATATYPES[dataType]
                value = d['value']
                                    
                self.Debug("    protocol: %s, id: %d, datatype: %s, value: %s, last updated: %s" % (sensResponse['protocol'], sensorId, dataType, value, lastUpdStr))
            
                if not sensData.has_key(liveSensorId):
                    sensData[liveSensorId] = []
                sensData[liveSensorId].append([sensResponse['protocol'], sensorId, dataType, value])

        dataresult = {}

        for cmd in cmds:
            if not self.DATAPOINTS.has_key(cmd):
                self.Log("ERROR: Undefined data point: " + cmd)
                continue

            if self.livePairs.has_key(cmd):
                sensId = self.livePairs[cmd]
                if sensData.has_key(sensId):
                    item = self.DATAPOINTS[cmd]
                    data = sensData[sensId]
                                
                    protocol = item[0]
                    model = item[1] # IGNORED
                    id = item[2]
                    datatype = item[3]                    

                    found = False
                    for d in data:
                        if d[0] == protocol and d[1] == id and d[2] == datatype:
                            found = True
                            dataresult[cmd] = d[3]
                            break
            
                    if not found:
                        self.Log("ERROR: Sensor data not found: " + cmd)
            
        return dataresult

    def liveApiRequest(self, method, params):
        global oauth
        if oauth == None:
            try:
                import oauth.oauth as oauth
            except:
                self.Log("ERROR: Unable to import oauth library. Install python-oauth.") 
        consumer = oauth.OAuthConsumer(self.publicKey, self.privateKey)
        token = oauth.OAuthToken(self.authToken, self.authTokenSecret)
        serviceUrl = "http://" + TELLDUS_LIVE_API_NAME + "/json/" + method

        oauthRequest = oauth.OAuthRequest.from_consumer_and_token(consumer, token=token, http_method='GET', http_url=serviceUrl, parameters=params)
        oauthRequest.sign_request(oauth.OAuthSignatureMethod_HMAC_SHA1(), consumer, token)
        headers = oauthRequest.to_header()
        headers['Content-Type'] = 'application/x-www-form-urlencoded'
        conn = httplib.HTTPConnection(TELLDUS_LIVE_API_NAME, 80)
        conn.request('GET', "/json/" + method + "?" + urllib.urlencode(params, True).replace('+', '%20'), headers=headers)
        response = conn.getresponse()
        return json.load(response)

        

###########################################################################

# Functions
