#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            storeFile.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.0f
#
# Date:            01.02.2016
#
# Description:     Data Store class to store logged data values into a
#                  file.
#                  
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.3)
# 
# Version history: ** 24.02.2009 v0.9a (Olli Lammi) **
#                  First beta version for testers. 
#
#                  ** 25.02.2009 v0.9c (Olli Lammi) **
#                  Logging and import changes 
#
#                  ** 26.02.2009 v0.9d (Olli Lammi) **
#                  Support for patterns in configuration. 
#
#                  ** 06.04.2009 v1.0a (Olli Lammi) **
#                  Changed to support configurable init. 
#
#                  ** 10.02.2011 v1.0b (Olli Lammi) **
#                  Interface changes. 
#
#                  ** 26.10.2011 v1.0c (Olli Lammi) **
#                  Added store filter init. 
#
#                  ** 07.01.2013 v1.0d (Olli Lammi) **
#                  Support for log time passing. 
#
#                  ** 24.03.2013 v1.0e (Olli Lammi) **
#                  Support for overwriting the log file. 
#
#                  ** 01.02.2016 v1.0f (Olli Lammi) **
#                  Added DELIMITER. 
#
###########################################################################

# Imports

import sys, os
import string, time
import shutil

from modules.core import store
from modules.core import configuration


###########################################################################

# Classes

class FileStore(store.Store):
    def __init__(self, modname):
        store.Store.__init__(self, modname)

    def handleConfiguration(self, conf):
        if not self.initStoreFilters(conf):
            return (0, "Invalid store filter.")
        
        self.filename = conf.getValue('NAME', '', self.getModuleName())
        self.overwrite = conf.isTrue('OVERWRITE', self.getModuleName())
        self.timepat = conf.getValue('TIMESTAMP', '', self.getModuleName())
        self.header = conf.getValue('HEADER', '', self.getModuleName())
        self.dataval = conf.getValue('DATAVALUE', '', self.getModuleName())
        self.footer = conf.getValue('FOOTER', '', self.getModuleName())
        self.delimiter = conf.getValue('DELIMITER', '', self.getModuleName())
        
        if len(self.filename) <= 0:
            return (0, "Missing file store filename.")
        return (1, '')

    def initConfiguration(self):
        if len(self.filename) <= 0:
            return (-1, "Missing file store filename.")
        
        status = 0
        try: 
            outf = open(self.filename, 'a')
            outf.close()
            status = 1
        except:
            status = 0
        if status:
            return (1, '')
        else:
            return (0, 'FileStore: Error opening file store: ' + self.filename)

    def insertDataImpl(self, timeval, values):
        tstr = ''
        headerstr = ''
        footerstr = ''
        valstr = ''
                
        try:
            tstr = time.strftime(self.timepat, time.localtime(timeval))
        except:
            self.Log("ERROR: Error creating timestamp using pattern: " + self.timepat)
            
        headerstr = ReplacePatterns(self.header, tstr, '', '')
        footerstr = ReplacePatterns(self.footer, tstr, '', '')
        delimstr = ReplacePatterns(self.delimiter, tstr, '', '')
        for item in values:
            if len(valstr) > 0:
                valstr = valstr + delimstr
            valstr = valstr + ReplacePatterns(self.dataval, tstr, item[0], item[1])
        
        self.Debug("Inserting data to file: " + self.filename)
        try:
            if self.overwrite:
                # create temporary file and replace the old file with it
                outf = open(self.filename + '_TeMpStAgE', 'w')
                outf.write(headerstr + valstr + footerstr)
                outf.close()
                shutil.move(outf.name, self.filename)
            else: 
                outf = open(self.filename, 'a')
                outf.write(headerstr + valstr + footerstr)
                outf.close()
        except Exception, e:
            self.Log("ERROR: Error writing data store file (%s): %s" % (e.__str__(), self.filename))
            return 0
        return 1

    @staticmethod
    def getModuleTypeName():
        return 'FILESTORE'
    
    @staticmethod
    def getAllowedConfigurationKeys():
        return (['NAME', 'OVERWRITE', 'TIMESTAMP', 'HEADER', 'DATAVALUE', 'FOOTER', 'DELIMITER'], [])


###########################################################################

# Functions

def ReplacePatterns(pattern, timestr, key, data):
    temps = pattern
    temps = ReplaceStrCI(temps, '%S%', ' ')
    temps = ReplaceStrCI(temps, '%N%', '\n')
    temps = ReplaceStrCI(temps, '%T%', '\t')
    temps = ReplaceStrCI(temps, '%D%', '\x0D')
    temps = ReplaceStrCI(temps, '%A%', '\x0A')
    temps = ReplaceStrCI(temps, '%TIME%', timestr)
    temps = ReplaceStrCI(temps, '%KEY%', key)
    temps = ReplaceStrCI(temps, '%VALUE%', data)
    temps = ReplaceStrCI(temps, '%VALUE_C%', string.replace(data, '.', ','))
    temps = ReplaceStrCI(temps, '%VALUE_P%', string.replace(data, ',', '.'))
    return temps

def ReplaceStrCI(str, pat, wstr):
    ind = 0 
    tstr = str
    while ind >= 0:
        ind = string.find(string.upper(tstr), string.upper(pat))
        if ind >= 0:
           tstr = tstr[:ind] + wstr + tstr[ind+len(pat):] 
    return tstr
