#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            storeRrd.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.0e
#
# Date:            20.01.2013
#
# Description:     Data Store class to store logged data values into a
#                  rrd database.
#                  
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.4)
# 
# Version history: ** 24.02.2009 v0.9b (Olli Lammi) **
#                  First version. 
#
#                  ** 25.02.2009 v0.9c (Olli Lammi) **
#                  Logging changes. Changed not to use rrdtool library
#                  because it requires external lib.  
#
#                  ** 06.04.2009 v1.0a (Olli Lammi) **
#                  Changed to support configurable init. 
#
#                  ** 10.02.2011 v1.0b (Olli Lammi) **
#                  Interface changes. 
#
#                  ** 26.10.2011 v1.0c (Olli Lammi) **
#                  Added store filter init. 
#
#                  ** 07.01.2013 v1.0d (Olli Lammi) **
#                  Support for log time passing. 
#
#                  ** 20.01.2013 v1.0e (Olli Lammi) **
#                  Timeoutable call. 
#
###########################################################################

# Imports

import sys, os 
import string, time
import subprocess

from modules.core import store
from modules.core import configuration
from modules.utils import utils


###########################################################################

# Classes

class RrdStore(store.Store):
    def __init__(self, modname):
        store.Store.__init__(self, modname)

    def handleConfiguration(self, conf):
        if not self.initStoreFilters(conf):
            return (0, "Invalid store filter.")
        
        self.RRDTOOL = conf.getValue('RRDTOOL_LOCATION', '', self.getModuleName())
        self.DB_FNAME = conf.getValue('DB_FILENAME', '', self.getModuleName())
        return (1, '')

    def initConfiguration(self):
        # check the existence of the RRDtool
        if not os.access(self.RRDTOOL, os.X_OK):
            return (0, 'RrdStore: Unable to find or execute RRDTool executable: ' + self.RRDTOOL)
        if not os.access(self.DB_FNAME, os.W_OK):
            return (0, 'RrdStore: Unable to find rrd db file or it is not writable: ' + self.DB_FNAME)
        return (1, '')

    def insertDataImpl(self, timeval, values):
        templatestr = ''
        valuestr = "%d" % (timeval, )
        for item in values:
            if len(templatestr) > 0:
                templatestr = templatestr + ':'
            templatestr = templatestr + string.strip(item[0])
            valuestr = valuestr + ':' + string.strip(item[1])
            
        rrdtoolcmd = self.RRDTOOL + ' update ' + self.DB_FNAME + ' -t ' + templatestr + ' ' + valuestr
        self.Debug("Inserting data to rrd database: " + self.DB_FNAME + " using command: " + rrdtoolcmd)
        
        tocmd = utils.TimeoutableShellCommand()
        tocmd.setTimeout(120)
        (stat, outstdout, outstderr) = tocmd.execute(rrdtoolcmd)
        
        if stat != 0:
            temps = "ERROR: Error writing data store RRD database: " + self.DB_FNAME + "\n"
            temps = temps + "  RRDTool command: " + rrdtoolcmd + "\n"
            temps = temps + "  RRDTool output: " + outstdout + outstderr
            self.Log(temps)
            return 0
        return 1                

    @staticmethod
    def getModuleTypeName():
        return 'RRD'
        
    @staticmethod
    def getAllowedConfigurationKeys():
        return (['RRDTOOL_LOCATION', 'DB_FILENAME'], [])
