#!env python
# -*- coding: iso-8859-1 -*-
###########################################################################
# 
# File:            storeThingSpeak.py
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         0.1a
#
# Date:            30.01.2017
#
# Description:     Data Store class to store logged data values into a
#                  ThingSpeak cloud (https://thingspeak.com).
#                  
# Requirements:    Python interpreter 2.4 or newer (www.python.org)
#                  (tested with 2.4.3)
# 
# Version history: ** 30.01.2017 v0.1a (Olli Lammi) **
#                  First beta version for testers. 
#
###########################################################################

# Imports

import sys, os
import string, time
import httplib, urllib


from modules.core import store
from modules.core import configuration

###########################################################################

# Constants

THINGSPEAK_FIELDS = ['field1', 'field2', 'field3', 'field4', \
                     'field5', 'field6', 'field7', 'field8']
    
###########################################################################

# Classes

class ThingSpeakStore(store.Store):
    def __init__(self, modname):
        store.Store.__init__(self, modname)

    def handleConfiguration(self, conf):
        if not self.initStoreFilters(conf):
            return (0, "Invalid store filter.")
        
        self.host = conf.getValue('HOST', '', self.getModuleName())
        self.url_path = conf.getValue('URL_PATH', '', self.getModuleName())
        self.apikey = conf.getValue('APIKEY', '', self.getModuleName())
        
        if len(self.host) <= 0:
            return (0, "Missing cloud service host.")

        if len(self.url_path) <= 0:
            return (0, "Missing cloud service URL path.")

        if len(self.apikey) <= 0:
            return (0, "Missing cloud service API key.")

        return (1, '')

    def initConfiguration(self):
        return (1, '')

    def insertDataImpl(self, timeval, values):
        tstr = time.strftime('%Y-%m-%dT%H:%M:%SZ', time.gmtime(timeval))
        
        params = {}
        params['api_key'] = self.apikey
        params['created_at'] = tstr

        headers = {}
        headers['Content-Type'] = 'application/x-www-form-urlencoded'

        hasData = False
        for val in values:
            if val[0] in THINGSPEAK_FIELDS:
                hasData = True
                params[val[0]] = val[1]
            else:
                self.Debug("Ignoring non ThingSpeak value field: %s" % val[0])

        if not hasData:
            self.Log("ERROR: No ThingSpeak fields found in data. Ignoring.")
            return 1

        self.Debug("Inserting data to ThingSpeak cloud: " + repr(params))    

        try:
            url_params = urllib.urlencode(params)
            conn = httplib.HTTPSConnection(self.host)
            conn.request("POST", self.url_path, url_params, headers)
            response = conn.getresponse()
            resdata = response.read()
            conn.close()
            if len(resdata) <= 0 or resdata == '0':
                self.Log("ERROR: Invalid response when inserting data: %s" % resdata)
                return 0
            else:
                self.Debug("Inserted data to cloud, received response: %s" % resdata)
        except Exception, e:
            self.Log("ERROR: Error updating data to ThingSpeak cloud: %s" % e.__str__())
            return 0
            
        return 1

    @staticmethod
    def getModuleTypeName():
        return 'THINGSPEAK'
    
    @staticmethod
    def getAllowedConfigurationKeys():
        return (['HOST', 'URL_PATH', 'APIKEY'], [])
